# =============================================================================
#
# File:         ubtbackupcron.tcl
# Project:      UBTool
#
# Started:      06.08.96
# Changed:      11.05.97
#
# Gilbert ROBERT (c) 1998


# =============================================================================
#
# Backup Procedure
#
# =============================================================================
proc backup { w Machine Device FileId FileMarker First} {
    global  ubt WarningErrors TotalMb CAT_SPAWN_ID

    set NbrMb 0
    set Date [exec date "+%h.%d.%y-%T"]

    writelog $FileId $w 0 "title" \
	    "\n*******************************************************\n" ""
    writelog $FileId $w 0 "config"\
	    "The date and time is:\t\t$Date" ""
    writelog $FileId $w 0 "config"\
	    "Backup on: \t\t\t$ubt(mach_todo_backup)" ""
    writelog $FileId $w 0 "config"\
	    "to the device/file: \t\t$ubt(tape_device)" ""
    writelog $FileId $w 0 "config"\
	    "As the operator: \t\t$ubt(username)" ""
    writelog $FileId $w 0 "config"\
	    "Using Local Command: \t$ubt(loc_backup_prog)\
	    $ubt(backup_options)" ""
    writelog $FileId $w 0 "config"\
	    "Using Remote Command: \t$ubt(rem_backup_prog)\
	    $ubt(backup_options)" ""
    writelog $FileId $w 0 "config" "\n\n" ""
    writelog $FileId $w $First "config"\
	    "FileMarker\tMachine\t\tFileSystem\tDate" ""
    writelog $FileId $w $First "config"\
	    "-------------------------------------------------------------" ""
    writelog $FileId $w 1 "config"\
	    "$FileMarker\t\t$Machine\t\t$Device\t$Date" ""
    writelog $FileId $w 0 "config" "\n" ""
    
    set Command ""
    
    # If the machine to do backup == hostname
    #
    if {[string compare $ubt(mach_todo_backup) $ubt(hostname)] == 0} {
	# If the machine to backup == hostname
	#
	if {[string compare $Machine $ubt(hostname)] == 0} {
	    set Command [concat  $ubt(wrapg)\
		    $ubt(loc_backup_prog) \
		    $ubt(backup_options) \
		    $ubt(tape_device) \
		    $Device ]
	} else {
	    # If the machine to backup != hostname
	    #
	    set Command [concat $ubt(rsh) $Machine -l $ubt(username) \
		    $ubt(wrapg) $ubt(rem_backup_prog) \
		    $ubt(backup_options) \
		    $ubt(mach_todo_backup):$ubt(tape_device) \
		    $Device ]
	}
    } else {   # If the machine to do backup != hostname
	#
	# If the machine to backup == hostname
	#
	if {[string compare $Machine $ubt(hostname)] == 0} {
	    set Command [concat  $ubt(wrapg) $ubt(rem_backup_prog) \
		    $ubt(backup_options) \
		    $ubt(mach_todo_backup):$ubt(tape_device)\
		    $Device]
	} else { 
	    # If the machine to backup != hostname
	    #
	    set Command [concat  $ubt(rsh) $Machine -l $ubt(username) \
		    $ubt(wrapg) $ubt(rem_backup_prog)\
		    $ubt(backup_options) \
		    $ubt(mach_todo_backup):$ubt(tape_device)\
		    $Device ]
	}
    }
    busy_on
    sleep 1
    # Has to be enough to allow a large dump to finish
    set timeout -1
    
    eval spawn -noecho $Command
    set  CAT_SPAWN_ID $spawn_id
    
    expect {
	-i $CAT_SPAWN_ID
	-re ".*DUMP IS DONE.*" { 
	    set Message $expect_out(buffer)
	    set Match  ""
	    regexp {([0-9\.]*)MB} $Message Match NbrMb
	    if { $Match != "" } {
		set TotalMb [expr $TotalMb + $NbrMb]
	    } else { set TotalMb [expr $TotalMb + 0.5] }
	    regexp {Dumping[^\(]*\(([^\)]*)\)} $Message \
		    Match FileSystem
	    writelog $FileId $w 2 "backup" \
		    "\n$Message" "\t$NbrMb Mb"
	}
	-re ".*The ENTIRE dump is aborted.*" {
	    set Message $expect_out(buffer)
	    writelog $FileId $w 2 "backup" "\n$Message\n" \
		    "$FileMarker\t\t$Machine\t\t$Device\t$Date\tFailed\n"
	    incr WarningErrors
	    wait -nowait
	    busy_off
	    return 1
	}
	-re ".*NEEDS ATTENTION: Is the new volume.*" {
	    writelog $FileId $w 1 "backup" \
		    "\nEnd of Media. Backup Aborted by operator\n" ""
	    wait -nowait
	    busy_off
	    return 2			
	}
	-re ".*NEEDS ATTENTION:.*"  {	
	    set Message $expect_out(buffer)
	    writelog $FileId $w 1 "backup" "\n$Message\n" ""
	    # Send a "no" message to the program
	    exp_send -i $CAT_SPAWN_ID "no\r"
	    incr WarningErrors
	    wait -nowait
	    busy_off
	    return 1
	}
	-re ".*ERROR: Exit status: 3.*"  {	
	    set Message $expect_out(buffer)
	    writelog $FileId $w 1 "backup" "\n$Message\n" ""
	    incr WarningErrors
	    wait -nowait
	    busy_off
	    return 1
	}
	-re ".*Connection timed out.*"  {	
	    set Message $expect_out(buffer)
	    writelog $FileId $w 0 "backup" "\n$Message\n" ""
	    incr WarningErrors
	    wait -nowait
	    busy_off
	    return 1
	}
	-re ".*not found.*"  {	
	    set Message $expect_out(buffer)
	    writelog $FileId $w 0 "backup" "\n$Message\n" ""
	    incr WarningErrors
	    wait -nowait
	    busy_off
	    return 1
	}
	default	{
	    writelog $FileId $w 1 "backup" \
		    "\nBackup Aborted by operator\n" ""
	    wait -nowait
	    busy_off
	    return 2
	}
    }
    wait -nowait
    busy_off
    return 0
}


# =============================================================================
# Backup Aborting
#
# =============================================================================
proc AbortBackup { } {
    global CAT_SPAWN_ID
	
    if {$CAT_SPAWN_ID != 0} {
	close $CAT_SPAWN_ID
    }
}

# =============================================================================
# Backup Lock
#
# =============================================================================
proc OpenLockBackup { } {
    global UBTLocked ubt

    if [file exists $ubt(lock_file)] {
	Warning "UBT has detected a lock file $ubt(lock_file)\n This may indicate that another user is running UBT." "Warning" 1
	set UBTLocked -1
    } elseif [catch  {open $ubt(lock_file) w} LockFileId] {
	Warning "Cannot open lock file $ubt(lock_file)" "Warning" 1
	set UBTLocked -1
    } else {
	close $LockFileId
	set UBTLocked 1
    }
}

proc CloseLockBackup { } {
    global UBTLocked ubt

    exec rm -f $ubt(lock_file)
    set UBTLocked 0
}


# =============================================================================
#
# Backup Processing
#
# =============================================================================
proc RunBackup { w } {
    global ubt 
    global NbrFS TabNew append WarningErrors Date ConfigPred
    global TotalMb Id_backc Id_backe Id_backs UBTLocked
    
    set FileMarker	0
    set WarningErrors   0
    set First	        1
    set Corr	        0
    set NCorr	        0
    set TotalMb	        0.0

    OpenLockBackup    
    if {$UBTLocked == 1} {
	if ![verif_config] {
	    if [catch  {open $ubt(log_file) $append} FileId] {
		if [catch {open $ubt(log_file) w } FileId] {
		    Warning "Cannot open $ubt(log_file)" "Warning" 1
		    CloseLockBackup
		    return 1
		}
	    }
	    # Tape Action  before the backup
	    #
	    
	    if [tape_action $ubt(tape_device) $ubt(TapeBeforeAction) $w] {
		CloseLockBackup
		return 1
	    }
	    
	    writelog $FileId $w 1 "title" "\n\n\t\t\tBACKUP  $Date\n" ""
	    
	    #  Wait Before Start
	    #
	    Warning "Backup starting at time: \
$ubt(TimeToStart)\n\tPlease wait ..\n"\
		    "Warning" 0
	    sleep [TimeWait $ubt(TimeToStart)]
	    foreach machine [array names TabNew] {
		set Tablist $TabNew($machine)
		for {set i 0} {$i < [llength $Tablist]} {incr i} {
		   set ret [backup $w $machine [lindex $Tablist $i] \
				$FileId $FileMarker $First]
		   if { $ret == 2} {
		       CloseLockBackup
		       return 1
		   }
		   set First 0
		   incr FileMarker
		   if { $ret == 1} {
		       incr NCorr
		   } else {
		       incr Corr
		   }
		}
		sleep $ubt(backup_wait)
	    }
	    writelog $FileId $w 0 "title" \
	      "\n\n*******************************************************" ""
	    writelog $FileId $w 1 "title"\
		    "\n\t\tEnd Of Backup $Date\n\t\tErrors:  $WarningErrors\
		    \n\t\tTotal Recorded: $TotalMb Mb" ""
	   writelog $FileId $w 0 "title" \
	      "*******************************************************\n" ""
	    close $FileId
	    if [tape_action $ubt(tape_device) $ubt(TapeAfterAction) $w] {
		CloseLockBackup
		return 1
	    }
	}
	CloseLockBackup
	return 0
    } else {
	Warning "Tape Device locked ..." "Warning" 0
    }
}
