# =============================================================================
#
# File:         ubtmain.tcl
# Project:      UBTool
#
# Started:      06.08.95
# Changed:      25.09.97
#
# Gilbert ROBERT (c) 1997


# =============================================================================
#
# Create Canvas Status
#
# =============================================================================
proc UpdateCanvas {ID Percent IDs MbSize}  {
	.backup.c.cv itemconfigure $ID -extent $Percent	
	if {$IDs != 0} {
		.backup.c.cv itemconfigure $IDs \
			-text "Backup Size: $MbSize Mb"
	}
}

proc PrintStatus { message } {
    global Id_status

    .menu.status itemconfigure $Id_status \
			-text "                                      "
    .menu.status itemconfigure $Id_status \
			-text "Status: $message"
}

# =============================================================================
#
# Write out message in text widget and/or in the log file
#
# =============================================================================
proc writelog { filelog textlog all tag text extra} {
	if { $all == 1 } { puts $filelog $text }
	if { $all == 2 } { puts $filelog $extra }
	flush $filelog
	$textlog insert end "$text\n" $tag
}

# =============================================================================
#
# Actions for Backup Mode Buttons 
#
# =============================================================================
proc SettingBackupMode {} {
	
    global ConfigPred Date ubt UBTLocked
    
    set ConfigPred 0
    set UBTLocked 0
    if {[string compare $ubt(BackupMode) "file"] == 0} {ReadPathFile}
    
    set ubt(log_file) ""
    append ubt(log_file) \
	    $ubt(BackupLog)  \
	    $ubt(log_prefix) \
	    $ubt(BackupMode) \
	    $Date
    if [winfo exist .set] { 
	.set.option.log.file.entry delete 0 end
	.set.option.log.file.entry insert 0 $ubt(log_file)
    }
}

# =============================================================================
# Create Main Window
#
# =============================================================================
proc CreateMainWindow {} {
	
	global ubt Date font bg fg
	global Id_backc Id_backe Id_backs Id_status TypeList 


	# Menu 
	#
	frame .menu \
		-relief raised \
		-borderwidth 1 \
		-bg $bg(Menu)

	button .menu.cy -command copyright \
		-font $font(Command)   \
		-text "Copyright" \
		-relief raised \
		-background $bg(Menu)   \
		-foreground $fg(Button) \

	button .menu.exit  -command "CloseLockBackup ; destroy ." \
		-background $bg(Menu) \
		-font $font(Command)  \
		-foreground $fg(ButtonStrong) \
		-borderwidth 2  \
		-text "Exit"    \
		-relief raised
    
	button .menu.abort  -command AbortBackup \
		-background $bg(Menu) \
		-font $font(Command)  \
		-foreground $fg(ButtonStrong) \
		-borderwidth 2  \
		-text "Abort Backup" \
		-relief raised
    
	canvas .menu.status \
		-width 8c \
		-height 0.8c \
		-bg $bg(Status)

        set Id_status [.menu.status create text 0.2c 0.6c \
		-text "Status: " \
		-fill $fg(Status) \
		-font $font(Status) \
		-anchor sw ]

	button .menu.save -command {SaveText .t.log} \
		-font $font(Command) \
		-text "Save Log" \
		-relief raised \
		-background $bg(Menu) \
		-foreground $fg(Button)

	button .menu.clean -command {ClearText .t.log} \
		-font $font(Command) \
		-text "Clear Log" \
		-relief raised \
		-background $bg(Menu) \
		-foreground $fg(Button)

	pack .menu.exit .menu.abort  -side left
	pack .menu.status -side left -expand true -fill x
	pack .menu.cy .menu.save .menu.clean -side right 

	# Pack Menu part (Bottom)
	#
	pack .menu -side bottom -fill both

        # Banner Frame
        #
        frame .banner \
		-relief raised \
		-borderwidth 2 \
		-bg $bg(Banner)

	# Image Print
	# 
	image create photo image1a -file $ubt(BackupIcon)/Issco_red.gif
	label .banner.image1 \
		-image image1a \
		-bg $bg(Banner)
        label .banner.image2 \
		-font $font(Banner) \
		-text "UNIX BACKUP TOOL" \
		-fg $fg(Banner) \
		-bg $bg(Banner)

        menubutton .banner.mbfile \
		-text "File" \
		-bg $bg(Banner) \
		-menu .banner.mbfile.menu
        menubutton .banner.mbopt \
		-text "Options" \
		-bg $bg(Banner) \
		-menu .banner.mbopt.menu

        set mbfile [menu .banner.mbfile.menu -tearoff 1]
        set mbopt  [menu .banner.mbopt.menu  -tearoff 1]

        $mbfile add command -label New  -command {puts "New file" }
	$mbfile add command -label Open -command {puts "Open file" }
	$mbfile add separator

        $mbopt add command -label Font  -command {puts "New Font" }
	$mbopt add command -label Size  -command {puts "New Size" }
	$mbopt add separator

        pack .banner.mbfile .banner.mbopt -side left -padx 2
	pack .banner.image1 -side right -padx 10 -pady 1
	pack .banner.image2 -side right -padx 10 -pady 1


	# Pack Banner part (Top)
	#
        pack .banner -side top -fill x -padx 0

	# Backup frame
	#
	frame .backup -relief raised \
		-borderwidth 2 \
		-bg $bg(Menu)
		
	# The list of backup modes
	#
	frame .backup.mode -relief raised \
		-borderwidth 2 \
		-bg $bg(Menu)
	label .backup.mode.label \
		-text "Backup Mode" \
		-font $font(Label) \
		-fg $fg(Label) \
		-bg $bg(Menu)

	pack .backup.mode.label -side top
	
	foreach el $TypeList {
		set BackupM [string range [file extension $el] 1 end]
		set WindowPathName ".backup.mode."
		append WindowPathName $BackupM
		radiobutton $WindowPathName \
		    -command {SettingBackupMode} \
		    -font $font(Mode) \
	            -variable ubt(BackupMode) \
		    -text $BackupM  \
		    -value $BackupM \
		    -highlightthickness 0 \
		    -bg $bg(Menu)
		pack $WindowPathName -side top -anchor w
	}

	append ubt(log_file)\
		$ubt(BackupLog) \
		$ubt(log_prefix)\
		$ubt(BackupMode)\
		$Date

	# The file mode
	radiobutton .backup.mode.file \
		-command {SettingBackupMode } \
		-font $font(Mode) \
		-variable ubt(BackupMode) \
		-text "file" \
		-bg $bg(Menu) \
		-value "file" -highlightthickness 0

	pack .backup.mode.file -side top -anchor w
	pack .backup.mode -side top -fill both  -ipady 10
	
	# Command

	frame .backup.command \
		-relief raised \
		-borderwidth 2 \
		-bg $bg(Menu)

	label  .backup.command.label \
		-text "Commands" \
		-font $font(Label) \
		-fg $fg(Label) \
		-bg $bg(Menu)

	button .backup.command.conf  -command ReadConfFile \
		-foreground $fg(ButtonStrong) \
		-font $font(Command) \
		-text "Configuration" \
		-relief groove \
		-bg $bg(Menu)
	button .backup.command.run   -command {RunBackup .t.log} \
		-font $font(Command) \
		-text "Run Backup" \
		-relief groove \
		-bg $bg(Menu)
#	button .backup.command.rest   -command Restore  \
#		-font $font(Command) \
#		-text "Restore" \
#		-relief groove \
#		-bg $bg(Menu)
	button .backup.command.rew   -command { tape_command } \
		-font $font(Command) \
		-text "Tape Commands" \
		-relief groove  \
		-bg $bg(Menu)

	pack .backup.command.label -side top
	pack .backup.command.conf -fill x -padx 20
	pack .backup.command.run -fill x  -padx 20
#	pack .backup.command.rest -fill x -padx 20
	pack .backup.command.rew -fill x  -padx 20
	pack .backup.command -side top -fill both -padx 10  -pady 5 -ipady 5


	# Canvas Part
	#
	set cv .backup.c
	frame $cv -relief raised \
		-borderwidth 2 \
		-bg $bg(Menu)

	canvas $cv.cv \
		-width 5c \
		-height 3c \
		-bg $bg(Menu)
	pack $cv.cv -in $cv -side left -fill both -expand true
	pack $cv -side top -fill both  -ipady 10

	set Id_backc [eval $cv.cv \
		create arc 0.6c 0.2c 1.6c 1.2c \
		-start 90 \
		-extent 0 \
		-style pieslice \
		-fill blue \
		-outline black]
	set Id_backe [ eval $cv.cv \
		create arc 3c 0.2c 4c 1.2c \
		-start 90 \
		-extent 0 \
		-style pieslice \
		-fill black \
		-outline black]

	.backup.c.cv create oval 0.6c 0.2c 1.6c 1.2c \
		-width 1 \
		-outline blue
	.backup.c.cv create oval 3c 0.2c 4c 1.2c \
		-width 1 \
		-outline red

	.backup.c.cv create text 1.1c 1.5c \
		-text "Done" \
		-justify center \
		-font $font(Canvas) 
	.backup.c.cv create text 3.5c 1.5c \
		-text "Errors" \
		-justify center \
		-font $font(Canvas) 

	set Id_backs [.backup.c.cv create text 2.2c 2.2c \
		-font $font(BackupSize) \
		-text "Backup Size: 0 Mb" \
		-justify center ]

	# Pack Backup part (left)
	#
	pack .backup -side left -fill both

	# Text Log
	#
	frame .t -relief raised \
		-borderwidth 2 \
		-bg $bg(Log)

	label .t.label \
		-font $font(Label) \
		-text "Backup Log" \
		-fg $fg(Label) \
		-bg $bg(Log)
	
	pack .t.label -side top

	text .t.log -wrap word \
		-bg LightSkyBlue2 \
		-fg orange4 \
		-font $font(Status) \
		-height 44  \
		-yscrollcommand ".t.s set"

	scrollbar .t.s -orient vertical -command {.t.log yview} 
	pack .t.s -side right -fill y -expand true
	pack .t.log -side left -fill both -expand true

	# Configure the tags font available for the text part
	#
	.t.log tag configure title \
		-font $font(LogTitle) \
		-relief sunken \
		-foreground black \
		-lmargin1 1c -lmargin2 1c
	.t.log tag configure config \
		-font $font(LogConfig) \
		-lmargin1 1c -lmargin2 1c \
		-foreground black
	.t.log tag configure backup \
		-font $font(LogBackup)\
		-foreground black \
		-lmargin1 1c -lmargin2 1c


	# Pack Text part (right)
	#
	pack .t -side left -expand true

}

