# =============================================================================
#
# File:         ubtmt.tcl
# Project:      UBTool
#
# Started:      06.08.95
# Changed:      26.08.97
#
# magnetic tape control
#
# Arguments:
#
# =============================================================================
 

proc tape_command {} {
    global ubt font bg fg

    if ![winfo exist .tape] { 
	toplevel .tape 
	wm title .tape "Tape Commands"
	
	set action ""
	frame .tape.options \
		-borderwidth 1 \
		-bg $bg(Menu)
	
	# Device
	frame .tape.options.device \
		-borderwidth 0 \
		-bg $bg(Menu)

	label .tape.options.device.label \
		-text "device" \
		-font $font(Label)\
		-fg $fg(Label)\
		-bg $bg(Menu)

	entry .tape.options.device.entry \
		-width 20 \
		-font $font(Entry) \
		-bg $bg(Entry) \
		-fg $fg(Entry)

	pack .tape.options.device.label .tape.options.device.entry -side left \
		-padx 1m -pady 2m
	pack .tape.options.device -side top -fill both -padx 10 -pady 10
	.tape.options.device.entry insert 0 $ubt(tape_device)

	# Argument
	frame .tape.options.arg \
		-borderwidth 0 \
		-bg $bg(Menu)

	label .tape.options.arg.label -text "Argument" \
		-font $font(Label) \
		-fg $fg(Label) \
                -bg $bg(Menu)
 
	entry .tape.options.arg.entry \
		-width 20 \
		-font $font(Entry) \
		-bg $bg(Entry) \
		-fg $fg(Entry)

	pack .tape.options.arg.label .tape.options.arg.entry -side left \
		-padx 1m -pady 2m
	pack .tape.options.device .tape.options.arg -side left \
		-padx 1m -pady 2m
	pack .tape.options -side top -fill both

	# Actions 
	frame .tape.command \
		-relief raised \
                -borderwidth 2 \
                -bg $bg(Menu)

	label .tape.command.label \
		-text "Action" \
		-font $font(Label) \
		-fg $fg(Label) \
                -bg $bg(Menu)

	radiobutton .tape.command.eof \
		-text "\[EOF\] Write count EOF marks at the current  position on the tape" \
		-variable action \
		-value eof \
                -font $font(Command) \
                -relief groove \
                -bg $bg(Menu) \
		-anchor w
	radiobutton .tape.command.fsf \
		-text "\[FSF\] Forward space over count EOF marks" \
		-variable action \
		-value fsf \
                -font $font(Command) \
                -relief groove \
                -bg $bg(Menu) \
		-anchor w 
	radiobutton .tape.command.fsr \
		-text "\[FSR\] Forward space count records" \
		-variable action \
		-value fsr \
                -font $font(Command) \
                -relief groove \
                -bg $bg(Menu) \
		-anchor w
	radiobutton .tape.command.bsf \
		-text "\[BSF\] Back space over count EOF marks" \
		-variable action \
		-value bsf \
                -font $font(Command) \
                -relief groove \
                -bg $bg(Menu) \
		-anchor w
	radiobutton .tape.command.bsr \
		-text "\[BSR\] Back space count records"\
		-variable action \
		-value bsr \
                -font $font(Command) \
                -relief groove \
                -bg $bg(Menu) \
		-anchor w
	radiobutton .tape.command.eom \
		-text "\[EOM\] Space to the end of recorded media on the tape"\
		-value eom \
                -font $font(Command) \
                -relief groove \
                -bg $bg(Menu) \
		-anchor w 
	radiobutton .tape.command.rewind \
		-text "\[REWIND\] Rewind the tape" \
		-variable action \
		-value rewind \
                -font $font(Command) \
                -relief groove \
                -bg $bg(Menu) \
		-anchor w
	radiobutton .tape.command.offline \
		-text "\[OFFLINE\] Rewind the tape and, unloading the tape" \
		-variable action \
		-value offline \
                -font $font(Command) \
                -relief groove \
                -bg $bg(Menu) \
		-anchor w
	radiobutton .tape.command.status \
		-text "\[STATUS\] Print status information about the tape unit"\
		-variable action \
                -font $font(Command) \
                -relief groove \
                -bg $bg(Menu) \
		-value status \
		-anchor w

	pack .tape.command.label -fill x -padx 10
	pack .tape.command.eof -fill x -padx 10
	pack .tape.command.fsf -fill x -padx 10
	pack .tape.command.fsr -fill x -padx 10
	pack .tape.command.bsf -fill x -padx 10
	pack .tape.command.bsr -fill x -padx 10
	pack .tape.command.eom -fill x -padx 10
	pack .tape.command.rewind -fill x -padx 10
	pack .tape.command.offline -fill x -padx 10
	pack .tape.command.status -fill x -padx 10
	pack .tape.command -side top -fill both 

	# Display
	frame .tape.display -relief raised -borderwidth 2
	text .tape.display.log -wrap word -bg LightSkyBlue2 -fg orange4\
		-height 10  -yscrollcommand ".tape.display.s set"
	scrollbar .tape.display.s -orient vertical \
		-command {.tape.display.log yview} 
	pack .tape.display.s -side right -fill y -expand true
	pack .tape.display.log -side left -fill both -expand true
	pack .tape.display -side top -expand true
	
	# Buttons
	frame .tape.button\
		-relief raised \
                -borderwidth 2 \
                -bg $bg(Menu)

	set Argument [.tape.options.arg.entry get]
	set Device   [.tape.options.device.entry get]
	if { $Argument == "" } {
	    set Arg $action
	} else {
	    set Arg [concat $action $Argument]
	}

	button .tape.button.act \
		-command { tape_button_push .tape $action} \
		-text "OK" -relief groove -borderwidth 1 \
                -font $font(Command) \
                -relief groove \
                -bg $bg(Menu)
	button .tape.button.cancel \
		-command { destroy .tape } \
		-text "Cancel" \
		-relief groove -borderwidth 1 \
                -font $font(Command) \
                -relief groove \
                -bg $bg(Menu)

	pack .tape.button.act .tape.button.cancel -side left
	pack .tape.button -side bottom -fill both
    }
}

proc tape_button_push { w action } {
    set Argument [$w.options.arg.entry get]
    set Device   [$w.options.device.entry get]
    if { $Argument == "" } {
	set Arg $action
    } else {
	set Arg [concat $action $Argument]
    }

    tape_action $Device $Arg $w.display.log
}

# =============================================================================
#      
#  Perform the action on the tape before or after backup.
#
# =============================================================================
proc tape_action { Device Argument w} {
	global ubt

	regsub -all \" $Argument \0 Arg
	if { ($Arg == "") || ($Device == "")} { 
	    Warning " Tape Arction: Device or arguments undefined !" \
		    "Warning" 1
	    return 1
	}
	# If the machine to do backup = hostname
	if {[string compare $ubt(mach_todo_backup) $ubt(hostname)] == 0} {
	    set Act [concat $ubt(wrapg) $ubt(tape_prog) \
			$ubt(tape_prog_opts) $Device $Arg]
	} else { 
	    set Act [concat $ubt(wrapg) $ubt(rsh) $ubt(mach_todo_backup) \
		     -l $ubt(username) $ubt(tape_prog) $ubt(tape_prog_opts) \
		     $Device $Arg]
	}
	PrintStatus "Action on tape: $Argument"
	# Action on tape
	busy_on
	sleep 1
	set timeout 180
	eval spawn $Act

	expect {
		-re "ERROR.*" {
			set Message $expect_out(buffer)
			$w insert end "ERROR: $Message $Arg\n"
			Warning "Couldn't execute \"$ubt(tape_prog)\" \
				on tape (machine: $ubt(mach_todo_backup)) " \
				"Error" 1
			return 1
		}
		-re "usage.*" {
			set Message $expect_out(buffer)
			$w insert end "ERROR: $Message $Arg\n"
			Warning "Couldn't execute \"$ubt(tape_prog)\" \
				on tape (machine: $ubt(mach_todo_backup)) " \
				"Error" 1
			return 1
		}
		timeout { 
		        Warning " Action on tape timeout ..." "Warning" 1
		        return 1
		}
		eof {
		        PrintStatus "End of action on tape: $Argument"
		}
		default { 
			set Message $expect_out(buffer)
			$w insert end "$Message\n"
			return 1 
		}
	}
	wait
	busy_off
	return 0
}
