# =============================================================================
#
# File:         ubtoptions.tcl
# Project:      UBTool
#
# Started:      06.08.95
# Changed:      26.08.97
#
# Gilbert ROBERT (c) 1997


# =============================================================================
#
#  Create the Options Window 
#
# =============================================================================
proc CreateOptionsWindow {} {

    global ubt Tab TabNew ConfigPred Bytes append font fg bg
 
    if ![winfo exist .set] { 
	toplevel .set 
	wm title .set "Unix Backup Tool Options"

	frame .set.option \
		-relief raised \
		-borderwidth 2 \
		-background $bg(Menu)

	set list .set.option.list

	frame $list \
		-relief raised \
		-borderwidth 2 \
		-background $bg(Menu)

	label $list.label \
		-text "Backup options" \
		-font $font(Label) \
		-fg $fg(Label) \
		-bg $bg(Label)

	pack $list.label -side top

  	labelentry $list.mach "Backup Machine" \
		25 \
		$ubt(mach_todo_backup)

	labelentry $list.device "Device" \
		25 \
		$ubt(tape_device)

  	labelentry $list.command "Backup Program" \
		25 \
		$ubt(loc_backup_prog)

	labelentry $list.rbackup "Remote Backup Program " \
		25 \
		$ubt(rem_backup_prog)

	labelentry $list.option "Backup Option " \
		25 \
		$ubt(backup_options)

	labelentry $list.rsh "Rsh Program" \
		25 \
		$ubt(rsh)

	labelentry $list.wait "Wait" \
		25 \
		$ubt(backup_wait)

	labelentry $list.mailprog "Mail Program" \
		25 \
		$ubt(mail_prog)

	labelentry $list.mailoptions "Mail Options" \
		25 \
		$ubt(mail_options) 

	labelentry $list.tapeprogram "Tape Program" \
		25 \
		$ubt(tape_prog)

	labelentry $list.tapebeforeaction \
		"Tape Action Before Backup" \
		25 \
		$ubt(TapeBeforeAction)

	labelentry $list.tapeafteraction \
		"Tape Action After Backup" \
		25 \
		$ubt(TapeAfterAction)

	labelentry $list.tapeprogoption \
		"Tape Program options " \
		25 \
		$ubt(tape_prog_opts)

	#
	# Save Log
	#
	set log .set.option.log

	frame $log -relief raised \
		-borderwidth 2 \
		-bg $bg(Menu)

	label $log.label -text "Output Log File" \
		-font $font(Label) \
		-bg $bg(Label) \
		-fg $fg(Label) 

	checkbutton $log.append \
		-offvalue "w" -onvalue  "a" \
		-variable append \
		-text "Append" \
		-relief flat \
		-font $font(Label) \
		-bg $bg(Label) \
		-fg $fg(Label) 


  	labelentry $log.file "File " \
		40 \
		$ubt(log_file)

	pack $log.label -side top
	pack $log.file $log.append -side left

	#
	# Time To Start
	# 

	set time .set.option.time

	frame $time \
		-relief raised \
		-borderwidth 2 \
		-bg $bg(Menu)

	label $time.label \
		-font $font(Label) \
		-text "Time to start" \
		-fg $fg(Label) \
		-bg $bg(Label)

  	labelentry $time.start "Start at " \
		26 \
		$ubt(TimeToStart)

	pack $time.label -side top
	pack $time.start -side left

	#
	# Bytes to Backup
	# 
	set byte .set.option.byte

	frame $byte \
		-relief raised \
		-borderwidth 2 \
		-bg $bg(Menu)

	label $byte.label -text "Byte to backup" \
		-font $font(Label) \
		-fg $fg(Label)\
		-bg $bg(Label) \

  	labelentry $byte.nbr "Total Bytes: " \
		22 \
		$Bytes

	pack $byte.label -side top
	pack $byte.nbr   -side left

	foreach tag {mach device command  rbackup option rsh wait mailprog \
		mailoptions  tapeprogram \
		tapebeforeaction tapeafteraction tapeprogoption} {
	    pack $list.$tag -side top -fill x
	}
	
	pack $list -side top -fill x
	pack $log -side top  -fill x
	pack $time -side top -fill x
	pack $byte -side top -fill x

	#
	# Print Machines & Devices
	# 

	set d .set.device

	frame $d \
		-relief raised \
		-borderwidth 2 \
		-bg $bg(Menu)

	label $d.label \
		-text "Machines & Devices" \
		-fg $fg(Label) \
		-bg $bg(Label) \
		-font $font(Label)

	scrollbar $d.s \
		-orient vertical \
		-command {.set.device.t yview} \
		-bg $bg(Label)

	pack $d.label -side top   -fill both
	pack $d.s     -side right -fill both

	text $d.t -yscrollcommand {.set.device.s set} \
		-wrap word \
		-width 60 -height 30 \
		-font $font(Device) \
		-setgrid true \
		-bg $bg(Log)

	pack $d.t -expand yes -fill both

	$d.t tag configure title \
		-font $font(LogTitle) \
		-foreground   $fg(LogTitle) \
		-relief sunken
	$d.t tag configure machine \
		-font $font(Machine) \
		-foreground   $fg(Machine) \
		-lmargin1 1c \
		-lmargin2 1c
	$d.t tag configure device \
		-foreground $fg(Device) \
		-font $font(Device)\

	if { $ConfigPred == 0 } { 
		set T  Tab 
	} else {  set T  TabNew }

	# Buttons definition
	# 
	set b .set.buttons
	frame  $b \
		-relief raised \
		-borderwidth 2 \
		-bg $bg(Banner)

	button $b.save \
		-command " GetSetting .set.device.t .set.option" \
		-text "Save" \
		-font $font(Command) \
		-bg $bg(Menu) \
		-fg $fg(ButtonStrong) \
		-relief raised

	button $b.exit -command "ExitOption .set .set.device.t .set.option" \
		-font $font(Command) \
		-bg $bg(Menu) \
		-fg $fg(ButtonStrong) \
		-borderwidth 5  \
		-text "Exit" \
		-relief raised

	button $b.addMach \
		-command {AddText .set.device.t "\nMachine " machine}\
		-text "New Machine" \
		-font $font(Command) \
		-bg $bg(Menu) \
		-fg $fg(Menu) \
		-fg $fg(Button)

	button $b.addFileSys \
		-command {AddText .set.device.t "\nFilesystem " device}\
		-text "New FileSystem" \
		-font $font(Command) \
		-bg $bg(Menu) \
		-foreground $fg(Button)
		 
	pack   $b.exit -side left -fill x -padx 5
	pack   $b.save -side left -fill x -padx 5

	pack   $b.addMach    -side right -fill x -padx 5
	pack   $b.addFileSys -side right -fill x -padx 5

	pack   $b -side right -fill both

	# Pack All
	pack	.set.buttons -side bottom -fill both
	pack	.set.option -side left -fill both
	pack	.set.device -side left -fill both	
   }

   $d.t delete 1.0 end   

   foreach index [array names T] {
	$d.t insert end "Machine $index\n" machine
	set Tablist $T($index)
	for {set i 0} {$i < [llength $Tablist]} {incr i} {
		$d.t insert end \
			"Filesystem\t[lindex $Tablist $i]\n" device
	}
    }
}

# =============================================================================
#
# Get Machine/Device and Options
#
# Arguments:
# t - the files system definition
# w - the options definition
#
# =============================================================================
proc GetSetting { t w } {

	global	ubt TabNew ConfigPred NbrFS

	.backup.command.conf configure -foreground black

	set NewMach	""
	set iend	[$t index end]
	set NbrFS	0.0
	set i		0
	set ConfigPred	1
	

	# Clean the Array TabNew
	foreach index [array names TabNew] {
		unset TabNew($index)
	}

	while { $i <= $iend } {
		set TextN [$t get $i.0 "$i.0 lineend"]
		regexp {^[	 ]*([^	 #]*)[	 ]*([^	 ]*)} \
			$TextN  test1 test2 test3
		if { $test3 != "" } {
		   case $test2 in {
			{Machine} { 
				set NewMach $test3 
				set TabNew($NewMach) ""
			}
			{Filesystem} {
			    set NbrFS [expr $NbrFS + 1.0]
			    lappend TabNew($NewMach) $test3
			}
		   }
		}
		incr i
	}
	#
	# Options Setting 
	# 
  	set ubt(mach_todo_backup) [$w.list.mach.entry get]
	set ubt(tape_device)      [$w.list.device.entry get]
  	set ubt(loc_backup_prog)  [$w.list.command.entry get]
	set ubt(backup_options)   [$w.list.option.entry get]
	set ubt(backup_wait)      [$w.list.wait.entry get]
	set ubt(mail_prog)        [$w.list.mailprog.entry get]
	set ubt(mail_options)     [$w.list.mailoptions.entry get]
	set ubt(rem_backup_prog)  [$w.list.rbackup.entry get]
	set ubt(rsh)              [$w.list.rsh.entry get]
	set ubt(tape_prog)        [$w.list.tapeprogram.entry get]
	set ubt(tape_prog_opts)   [$w.list.tapeprogoption.entry get]
  	set ubt(log_file)         [$w.log.file.entry get]
	set ubt(TimeToStart)      [$w.time.start.entry get]
	set ubt(TapeAfterAction)  [$w.list.tapeafteraction.entry get]
  	set ubt(TapeBeforeAction) [$w.list.tapebeforeaction.entry get]
}


# =============================================================================
#
# Read the first lines of the given configuration file to know:
#
#    - The machine list to backup
#    - The type of the backup
#
# =============================================================================
proc PreTextLoadFile {file} {
    global  MachineList ubt

    if [catch  {open $file} f ] {
		error "Cannot open $file"
    } 
    set Total	0

    while { [gets $f line] >=0 } {
    regexp {^[#	 ]*([^	 #]*)[	 ]*([^	#]*)[	 ]*([^	#]*)[	 ]*([^	#]*)}\
		$line  test1 test2 test3 test4 test5
	if { $test3 != "" } {
	   case $test2 in {
		{machines_to_backup} { set MachineList $test3 }
		{backup_type}	     { set ubt(BackupType)  $test3 }
	   }
       }
    }
    close $f
}

# =============================================================================
#
# From a given configuration file Read and Set the options
#
# Arguments:
# file - the configuration file
#
# =============================================================================
proc TextLoadFile {file} {
    global  Tab

    if [catch  {open $file} f ] {
		error "Cannot open $file"
    } 
    set Total	0
    # 
    # Clean the Array Tab & The window Text
    foreach index [array names Tab] {
	unset Tab($index)
    }
    ClearText .set.device.t

    while { [gets $f line] >=0 } {
    regexp {^[#	 ]*([^	 #]*)[	 ]*([^	#]*)[	 ]*([^	#]*)[	 ]*([^	#]*)}\
		$line  test1 test2 test3 test4 test5
    if { $test3 != "" } {
	case $test2 in {
	    {mach_todo_backup} { CleanSetEntry mach.entry $test3 }
	    {tape_device}      { CleanSetEntry device.entry $test3 }
	    {loc_backup_prog}  { CleanSetEntry command.entry $test3 }
	    {rem_backup_prog}  { CleanSetEntry rbackup.entry $test3 }
	    {backup_options}   { CleanSetEntry option.entry $test3 }
	    {backup_wait}      { CleanSetEntry wait.entry $test3 }
	    {mail_options}     { CleanSetEntry mailoptions.entry $test3 }
	    {mail_prog}	       { CleanSetEntry mailprog.entry $test3 }
	    {options}	       { CleanSetEntry option.entry $test3 }
	    {remote_shell}     { CleanSetEntry rsh.entry $test3 }
	    {tape_after_action}  { CleanSetEntry tapeafteraction.entry $test3}
	    {tape_before_action} { CleanSetEntry tapebeforeaction.entry $test3}
	    {tape_prog}	       { CleanSetEntry tapeprogram.entry $test3 }
	    {tape_prog_opts}   { CleanSetEntry tapeprogoption.entry $test3}
	    {log_file}	       { CleanSetEntry file.entry $test3 }
	    {mach_name}	       { set NewMach $test3
	                         set Tab($NewMach) "" }
	    {filesystem}       {
		if { $test5 != ""} {
		    set Len [expr [string length $test5] - 3]
		    incr Total [string range $test5 0 $Len]
		    set util "$test3\t$test4\t$test5"
		} else {
		    set util "$test3\t$test4"
		}
		   lappend Tab($NewMach) $util
		}  	
	   }
       }
    }
    close $f
    # Edit Machine & Device to Backup
    #
    foreach index [array names Tab] {
	.set.device.t insert end "Machine $index\n" machine
	set Tablist $Tab($index)
	for {set i 0} {$i < [llength $Tablist]} {incr i} {
	  .set.device.t insert end \
		"Filesystem\t[lindex $Tablist $i]\n" device
	}
    }
    .set.option.byte.nbr.entry delete 0 end
    set Total [expr $Total / 1000 ]
    .set.option.byte.nbr.entry insert 0 "$Total Mb"
}


# =============================================================================
#
#  Save And Exit Option Frame
#
#  Arguments
#  w       - the option frame descriptor
#  wtext   - the FS  window part
#  woption - the general options window part
# =============================================================================
proc ExitOption { w wtext woption} {

    global fg bg font
    global ConfigPred

	if { $ConfigPred == 1 } { 
	    destroy $w
	    return 0
	}
	toplevel .exit
	wm title .exit "Exit"

	frame .exit.buttons \
		-relief raised \
		-borderwidth 2 \
		-bg $bg(Menu)

	label .exit.bitmap \
		-bitmap warning \
		-font $font(Label) \
		-bg $bg(Label) \
		-fg $fg(Label)
	

	button .exit.buttons.quit \
		-command "destroy $w;destroy .exit" \
		-text "Exit" \
		-font $font(Command) \
		-bg $bg(Menu) \
		-fg $fg(Button)
	
	button .exit.buttons.savequit -command \
		"GetSetting $wtext $woption; destroy $w; destroy .exit" \
		-text "Save & Exit" \
		-font $font(Command) \
		-bg $bg(Menu) \
		-fg $fg(Button)
	
	pack .exit.buttons.quit .exit.buttons.savequit -side top \
	        -fill x 
	message .exit.msg \
		-width 10c \
		-justify center \
		-text " \
		Your options have not been saved !\n \
Save before to quit \n "

	pack .exit.bitmap -side left -padx 3m -pady 3m
	pack .exit.msg -side left
	pack .exit.buttons -side bottom
}


# =============================================================================#
# Clean  the Option and set a new value
#
# Arguments:
# name  - the name of the option
# value - the new value
#
# =============================================================================

proc CleanSetEntry {name value} { 
	.set.option.list.$name delete 0 end
	.set.option.list.$name insert 0 $value
}


