# ========================================================================
#
# File:         ubtutils.tcl
# Project:      UBTool
#
# Started:      06.08.96
# Changed:      26.08.97
#
# Gilbert ROBERT (c) 1997
 


# =========================================================================
#
# Time to wait before running the backup 
#
# Arguments:
# Time - The time when the backup has to be done
#
# =========================================================================
proc TimeWait { Time } {
	
   if {("$Time" == "Now") || ($Time == "") || ($Time == "now")} {
	return 0
   }
   regexp {^[	 ]*([^	 :]*):([^	 ]*)} $Time test1 Hour Minute
   if { "$Hour" != "" } {
        set Hour_r [exec date "+%H"]
        set Minut_r [exec date "+%M"]
	set HDiff [expr $Hour - $Hour_r ]
	set MDiff [expr $Minute - $Minut_r ]
	if { $HDiff < 0 } {
		incr HDiff 24
	}
	if { $MDiff < 0 } {
		set MDiff [expr 60 + $MDiff]
		if { $HDiff == 0 } {
			incr HDiff 24
		} 
		incr HDiff -1
	}
        set MinutesR  [expr $HDiff*60 + $MDiff]
	# in seconds
	return [expr $MinutesR * 60]
   } else { return 0 }
}


# ========================================================================
#
#  Return the Path File of Data
#
# ========================================================================
proc ReadPathFile {  } {
    global ubt

	toplevel .path
	wm title .path "Path"

	frame .path.path
  	labelentry .path.path.file "File " 50 $ubt(DefaultDataFile)

	frame .path.buttons
	label .path.label -text "File" -fg blue
	button .path.buttons.quit  -command "destroy .path" -text "Quit"
	button .path.buttons.apply -text "Ok" -command {  
	    set ubt(DefaultFile) [.path.path.file.entry get];destroy .path } 

	pack .path.label -side top
	pack .path.buttons.apply -side left -fill x -padx 5
	pack .path.buttons.quit -side right -fill x -padx 5
	pack .path.path.file -fill x
	pack .path.path .path.buttons  -side top  -expand yes -pady 3
	bind .path <Return> { 
	    set ubt(DefaultFile) [.path.path.file.entry get] }
}

# =========================================================================
#
# Create a label entry, and pack it
#
# =========================================================================
proc labelentry { path text length var } {

    global bg fg font

    frame $path \
	    -relief raised \
	    -borderwidth 1 \
	    -background $bg(Menu)

    label $path.label \
	    -text $text \
	    -font $font(Menu) \
	    -bg $bg(Menu) \
	    -fg $fg(Menu)

    entry $path.entry \
	    -width $length \
	    -relief sunken \
	    -font $font(Entry) \
	    -bg $bg(Entry) \
	    -fg $fg(Entry)

    pack $path.entry -side right
    pack $path.label -side left

    $path.entry insert 0  $var
}


# =========================================================================
# Create Backup Type List from data directory
#
# Read the Files with the prefix "data.tpl" in the 
# $BackupData directory
# =========================================================================

proc CreateType {} {
    global ubt TypeList
	
    set TypeList [split [glob -nocomplain "$ubt(BackupData)/data.tpl.*"]]
}
# ========================================================================
#
# Text Processing
#
# ========================================================================
proc ClearText { w } { 
    $w delete  1.0 end 
}

proc SaveText { w } {
    global ubt
    
    ReadPathFile
    if [catch  {open $ubt(DefaultFile) w } FileId] {
	error "Cannot open $ubt(DefaultFile)"
    }
    set Text [$w get 1.0 end]
    puts $FileId $Text
    flush $FileId
    close $FileId
}

proc AddText { w text tag } {
	set coord [$w index insert]
	$w insert $coord $text $tag
}


# manage busy state
 
proc busy_on {} {
        global Busy P
        if {![info exists Busy]} {
	    foreach win [winfo children .menu] {
		bindtags $win {busy [bindtags $win]}
	    }
	    foreach win [winfo children .backup.command] {
		bindtags $win {busy [bindtags $win]}
	    }
        }
        set Busy 1
        bind all <Button> {bell;break}
        update idletasks
}
 
proc busy_off {} {
    global Busy P
    if {![info exists Busy]} return
    foreach win [winfo children .backup.command] {
	bindtags $win {}
    }
    foreach win [winfo children .menu] {
	bindtags $win {}
    }
    set Busy 0
    bind all <Button> {}
    update idletasks
}
 
