/*
 * burt.h
 *
 *    This file defines constants and data structures used by Burt
 *
 * Copyright (C) 1995 - 1999 Eric Melski and Dean Jansa
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice and this permission notice appear in all copies,
 * and that all support documentation is included with each copy of the 
 * software.  A reasonable copying fee may be charged for any distribution of 
 * this software, but no fee may be charged for the software itself. You may 
 * distribute this software in aggregate with other programs as part of a 
 * larger software distribution provided that this software is not advertised 
 * as a product of your own.  Eric Melski and Dean Jansa make no 
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * ERIC MELSKI AND DEAN JANSA DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL ERIC MELSKI OR DEAN JANSA BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * $Id: burt.h,v 1.19 1999/04/15 18:42:22 jmelski Exp $
 */

#ifndef _BURT
#define _BURT
#define _POSIX_SOURCE

#define BURT_VERSION "2.4.7"
#define BURT_MAJOR_VERSION 2
#define BURT_MINOR_VERSION 4
#define BURT_REVISION      7
#define BURT_MAGIC_NUMBER 2470          /* BURT_MAGIC_NUMBER ==
					 *   BURT_MAJOR_VERSION * 1000 +
					 *   BURT_MINOR_VERSION * 100  +
					 *   BURT_REVISION      * 10
					 * (usually)
					 */

/* 
 * NOTE: If you change any of the values below this comment, you will not be 
 * able to read tapes written with old values.  Think carefully before making
 * any changes!  Consider yourself warned.
 */

#define BURT_MAX_HOSTLEN   256   /* Maximum host name length. */
#define BURT_MAX_ATOMLEN   256   /* Maximum atom name length. */
#define BURT_MAX_TYPELEN   32    /* I truly hope we all can find unique */
                                 /* dump types within 32 bytes...  :-)   */
#define BURT_MAX_LEVELLEN  16    /* Maximum level string length. */
#define BURT_MAX_LABELLEN  32    /* Max length of a tape label string. */
#define BURT_MAX_DATELEN   32    /* Max length of date string in tape label. */

/*
 * There seems to be conflicting ideas about which is better.  HP claims
 * 32Kb is the best for streaming a tape, and 65535 is the max blocksize
 * most tape drives can write in a atomic write.
 */

#define BURT_MAX_BLOCKSIZE 32768 /* Maximum size of a tape block. */
#ifdef BIGBLOCK
#define BURT_MAX_BLOCKSIZE 65535 /* Maximum size of a tape block. */
#endif

typedef struct Burt_Packet {
    int magic;
    char host[BURT_MAX_HOSTLEN]; /* Host on which atom lives. */
    char atom[BURT_MAX_ATOMLEN]; /* The filesys/volume (/usr,u.jmelski) */
    char type[BURT_MAX_TYPELEN]; /* User defined dump type - ufs, afs... */
    char level[BURT_MAX_LEVELLEN]; /* The level of this dump. */
    int blockNum;  /* Current block number. */
    int blockSize; /* Size of current block. */
    unsigned int totalSize; /* Sum of all block sizes for this host and atom */
    unsigned int checksum; /* Checksum of current data block */
    char buffer[BURT_MAX_BLOCKSIZE - 580];  /* stores packet data */
} Burt_Packet;


typedef struct Burt_TapeLabel {
  int  magic;
  char label[BURT_MAX_LABELLEN];  /* User supplied lable for tape. */
  char date[BURT_MAX_DATELEN];    /* Date tape was labeled. */
  char tape[3];                   /* Number of tape in multi tape series. */
  char padding[1024 - 72];        /* pad structure to 1024 bytes */
} Burt_TapeLabel;

typedef struct Burt_LoadNextTapeData {
  int tapeReady;
  Tcl_Channel mtChannel;
  Tcl_Channel logChannel;
  Tcl_TimerToken promptTimer;
  Tcl_TimerToken timer;
  char *prompt;
} Burt_LoadNextTapeData;

extern int Burt_Init(Tcl_Interp *interp);

#endif /* _BURT */
