/*
 * burtMain.c
 *
 *    This file implements the init function for burt.  This is where
 *    new Tcl commands are exported up to the user level.
 *
 * Copyright (C) 1995 - 1998 Eric Melski and Dean Jansa
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice and this permission notice appear in all copies,
 * and that all support documentation is included with each copy of the 
 * software.  A reasonable copying fee may be charged for any distribution of 
 * this software, but no fee may be charged for the software itself. You may 
 * distribute this software in aggregate with other programs as part of a 
 * larger software distribution provided that this software is not advertised 
 * as a product of your own.  Eric Melski and Dean Jansa make no 
 * representations about the suitability of this software for any purpose.  
 * It is provided "as is" without express or implied warranty.
 *
 * ERIC MELSKI AND DEAN JANSA DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL ERIC MELSKI OR DEAN JANSA BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 */


#define _POSIX_SOURCE
#ifndef lint
static char rcsid[] = "$Id: burtMain.c,v 1.16 1999/04/01 20:54:30 jmelski Exp $";
#endif

#include <burtInt.h>

/*
 *----------------------------------------------------------------------
 *
 * Burt_Init --
 *
 *	This procedure perform Burt-specific initialization.
 *
 * Results:
 *	Returns a standard Tcl completion code, and leaves an error
 *	message in interp->result if an error occurs.
 *
 * Side effects:
 *	Depends on the startup script.
 *
 *----------------------------------------------------------------------
 */

int 
Burt_Init(Tcl_Interp *interp)
{
    int code ;
    Tcl_CmdInfo cmdInfo;

    if (Tcl_PkgRequire(interp, "Tcl", TCL_VERSION, 1) == NULL) {
        return(TCL_ERROR);
    }

    BurtScheduleInit();
    BurtReadtapeInit();

    if (Tcl_GetCommandInfo(interp, "backup", &cmdInfo))
    {
      Tcl_AppendResult(interp, "command \"backup\" already installed", 
		       (char *)NULL);
      return(TCL_ERROR);
    }

    if (Tcl_GetCommandInfo(interp, "recover", &cmdInfo))
    {
      Tcl_AppendResult(interp, "command \"recover\" already installed",
		       (char *)NULL);
      return(TCL_ERROR);
    }

    if (Tcl_GetCommandInfo(interp, "schedule", &cmdInfo))
    {
      Tcl_AppendResult(interp, "command \"schedule\" already installed",
		       (char *)NULL);
      return(TCL_ERROR);
    }

    if (Tcl_GetCommandInfo(interp, "readtape", &cmdInfo))
    {
      Tcl_AppendResult(interp, "command \"readtape\" already installed",
		       (char *)NULL);
      return(TCL_ERROR);
    }

    if (Tcl_GetCommandInfo(interp, "status", &cmdInfo))
    {
      Tcl_AppendResult(interp, "command \"status\" already installed",
		       (char *)NULL);
      return(TCL_ERROR);
    }

    if (Tcl_GetCommandInfo(interp, "mt", &cmdInfo))
    {
      Tcl_AppendResult(interp, "command \"mt\" already installed",
		       (char *)NULL);
      return(TCL_ERROR);
    }

    Tcl_CreateObjCommand(interp, "backup", Burt_BackupObjCmd, NULL, NULL);
    Tcl_CreateCommand(interp, "recover", Burt_RecoverCmd, NULL, NULL);
    Tcl_CreateObjCommand(interp, "schedule", Burt_ScheduleObjCmd, NULL, NULL);
    Tcl_CreateCommand(interp, "readtape", Burt_ReadTapeCmd, NULL, NULL);
    Tcl_CreateCommand(interp, "status", Burt_StatusCmd, NULL, NULL);
    Tcl_CreateObjCommand(interp, "mt", Burt_MtObjCmd, NULL, NULL);
    Tcl_SetVar(interp, "burt_complete", "0", TCL_GLOBAL_ONLY);
    Tcl_SetVar(interp, "burt_version", BURT_VERSION, TCL_GLOBAL_ONLY);
    BurtStatusReset();

    /*
     * Specify a user-specific startup file to invoke if the application
     * is run interactively.  Typically the startup file is "~/.apprc"
     * where "app" is the name of the application.  If this line is deleted
     * then no user-specific startup file will be run under any conditions.
     */
    Tcl_SetVar(interp, "tcl_rcFileName", "~/.burtrc", TCL_GLOBAL_ONLY); 

    /* This will either return TCL_OK or TCL_ERROR */
    code = Tcl_PkgProvide(interp, "Burt", BURT_VERSION);
    return(code);
}


/*
 *----------------------------------------------------------------------
 *
 * GetMainVersion
 *     Returns a pointer to the rcsid string
 *
 * Results:
 *     A pointer to the rcsid string
 *
 *----------------------------------------------------------------------
 */
char *
GetMainVersion(void)
{
  return(rcsid);
}
