/*
 * burtRecover-event.c
 *
 *    This file implements the event driven recover command.  
 *    This version of recover is no longer used; in fact this code has not been
 *    kept up-to-date with the other code, because of bugs with Solaris and 
 *    having event driven tape reading.  However, I am keeping it around so that
 *    someday, hopefully, I can make recover event driven (the way it should be).
 *
 * Copyright (C) 1995 - 1998 Eric Melski and Dean Jansa
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice and this permission notice appear in all copies,
 * and that all support documentation is included with each copy of the software.
 * A reasonable copying fee may be charged for any distribution of this 
 * software, but no fee may be charged for the software itself. You may 
 * distribute this software in aggregate with other programs as part of a 
 * larger software distribution provided that this software is not advertised 
 * as a product of your own.  Eric Melski and Dean Jansa make no representations
 * about the suitability of this software for any purpose.  It is provided 
 * "as is" without express or implied warranty.
 *
 * ERIC MELSKI AND DEAN JANSA DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL ERIC MELSKI OR DEAN JANSA BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 */

#ifndef lint
static char rcsid[] = "$Id: burtRecover-event.c,v 1.3 1998/08/18 20:39:43 jmelski Exp $";
#endif

#include <burtInt.h>
#include <errno.h>

typedef struct Burt_RecoverControlBlock {
    Tcl_Interp *interp;
    Tcl_Channel inputChannel;     /* Channel to read data from. */
    Tcl_Channel outputChannel;    /* Channel to write output.   */
    Tcl_Channel logChannel;       /* Channel to write log data. */
    char host[BURT_MAX_HOSTLEN];  /* Host on which atom lives. */
    char atom[BURT_MAX_ATOMLEN];  /* The filesys/volume (/usr,u.djansa.backup) */
    char type[BURT_MAX_TYPELEN];  /* User defined dump type - ufs, afs... */
    char level[BURT_MAX_LEVELLEN];/* The level of this dump. */
    int lastBlock;
} Burt_RecoverControlBlock;

static int StartRecoverChannel(Tcl_Interp *interp, Tcl_Channel inputChannel, 
			       Tcl_Channel logChannel, char *host, char *atom,
			       char *level, char *type);
static int GetUserRecoverChannel(Tcl_Interp *interp, 
				 Burt_RecoverControlBlock *recoverCB);
static void ProcessRecoverInputChannel(ClientData clientData, int mask);
static int LoadNextRecoverTape(Tcl_Channel mtChannel, Tcl_Channel logChannel);




/*
 *----------------------------------------------------------------------
 *
 * Burt_RecoverCmd --
 *
 *	This procedure implements the Tcl command recover.
 *
 * Results:
 *      Returns a standard Tcl completion code, and leaves an error
 *	message in interp->result if an error occurs.
 *
 * Side effects:
 *	
 *
 *----------------------------------------------------------------------
 */

int 
Burt_RecoverCmd(ClientData cdata, Tcl_Interp *interp, int argc, char **argv)
{
    Tcl_Channel inputChannel;
    Tcl_Channel logChannel;
    int filemarkCount;
    int mode;
    char logBuffer[1024];
    char *level = NULL, *type = NULL;

    if ((argc < 5) || (argc > 8)) {
	Tcl_AppendResult(interp, "wrong # args: should be ", 
			 "\"recover inputchannel logchannel host atom ?level? ?type?",
			 "?skipfilemarks?\"", (char *) NULL);
	return TCL_ERROR;
    }
    
    inputChannel = Tcl_GetChannel(interp, argv[1], &mode);
    if( inputChannel == NULL ) {
	return(TCL_ERROR);
    }

    if ((mode & TCL_READABLE) == 0) {
	Tcl_UnregisterChannel(interp, inputChannel);
	Tcl_SetResult(interp, 
		      "input channel wasn't opened for reading",
		      TCL_STATIC);
	return(TCL_ERROR);
    }     

    logChannel = Tcl_GetChannel(interp, argv[2], &mode);
    if( logChannel == NULL ) {
	return(TCL_ERROR);
    }

    if ((mode & TCL_WRITABLE) == 0) {
	Tcl_UnregisterChannel(interp, logChannel);
	Tcl_SetResult(interp, 
		      "log channel wasn't opened for writing",
		      TCL_STATIC);
	return(TCL_ERROR);
    }     
 
    BurtMtReadLabel(inputChannel, NULL);
    BurtSetBackupTapes(1);

    if (BurtMtRewind(inputChannel) == MT_ERROR) {
	Tcl_SetResult(interp, "Could not rewind tape", TCL_STATIC);
	return(TCL_ERROR);
    }

    if (argc > 5)
	type = argv[5];
    
    if (argc > 6)
	level = argv[6];

    if (argc > 7) {
	if (Tcl_GetInt(interp, argv[7], &filemarkCount) != TCL_OK) {
	    return(TCL_ERROR);
	}
	
	sprintf(logBuffer, "BURT: Attempting to skip %d filemarks...\n", filemarkCount);
	Tcl_Write(logChannel, logBuffer, strlen(logBuffer));
	Tcl_Flush(logChannel);
	if (BurtMtFastForward(inputChannel, filemarkCount) == MT_ERROR) {
	    Tcl_UnregisterChannel(interp, inputChannel);
	    Tcl_SetResult(interp, 
			  "Fast Forward failed, try again without \"skipfilemarks\"", 
			  TCL_STATIC);
	    Tcl_Write(logChannel, "Fast Forward failed, try again without \"skipfilemarks\"", -1);
	    Tcl_Flush(logChannel);
	    return(TCL_ERROR);
	}
	sprintf(logBuffer, "BURT: Successfully skipped %d filemarks.\n", filemarkCount);
	Tcl_Write(logChannel, logBuffer, strlen(logBuffer));
	Tcl_Flush(logChannel);
    }

    Tcl_SetVar(interp, "burt_complete", "0", TCL_GLOBAL_ONLY);
    StartRecoverChannel(interp, inputChannel, logChannel, argv[3], argv[4], level, type);

    return(TCL_OK);
}



/*
 *----------------------------------------------------------------------
 *
 * StartRecoverChannel --
 *
 *	This procedure sets a recover control block with the information
 *      gathered from the user command and starts a channel handler to
 *      to read data from tape.
 *
 * Results:
 *      Returns TCL_OK if all went well, TCL_ERROR otherwise (and fills
 *      in the interp->result if there was an error).
 *
 * Side effects:
 *	
 *
 *----------------------------------------------------------------------
 */

static int
StartRecoverChannel(Tcl_Interp *interp, Tcl_Channel inputChannel, 
		    Tcl_Channel logChannel, char *host, char *atom, char *level, char *type)
{
    Burt_RecoverControlBlock *recoverCB;

    recoverCB = (Burt_RecoverControlBlock *)malloc(sizeof(Burt_RecoverControlBlock));
    if (recoverCB == NULL) {
	Tcl_SetResult(interp, "Out of Memory in StartRecoverChannel",
		      TCL_STATIC);
	return TCL_ERROR;
    }
    
    recoverCB->interp = interp;
    recoverCB->inputChannel = inputChannel;
    recoverCB->logChannel = logChannel;
    recoverCB->outputChannel = (Tcl_Channel) NULL;
    recoverCB->lastBlock = 0;
    strncpy(recoverCB->host, host, BURT_MAX_HOSTLEN);
    strncpy(recoverCB->atom, atom, BURT_MAX_ATOMLEN);
    if (level != NULL) strncpy(recoverCB->level, level, BURT_MAX_LEVELLEN);
    else strncpy(recoverCB->level, "", BURT_MAX_LEVELLEN);
    if (type != NULL) strncpy(recoverCB->type, type, BURT_MAX_TYPELEN);    
    else strncpy(recoverCB->type, "", BURT_MAX_TYPELEN);    

    Tcl_CreateChannelHandler(recoverCB->inputChannel, TCL_READABLE, 
			     ProcessRecoverInputChannel, (ClientData) recoverCB);

    return(TCL_OK);
}



/*
 *----------------------------------------------------------------------
 *
 * GetUserRecoverChannel --
 *
 *	This procedure fills in the outputChannel field of the recover 
 *      control block.  It does this by calling a user defined TCL procedure 
 *      (looked up in a Tcl array based on the "type" field of the recover
 *      control block.) which must return one writeable channel.  (This is 
 *      where the data which ends up going to the user defined proc comes 
 *      from).
 *
 * Results:
 *	Returns TCL_OK if the channels was gathered, TCL_ERROR otherwise.
 *      If TCL_ERROR is returned, the interp->result field is filled
 *      in with an error message.
 *
 * Side effects:
 *
 *----------------------------------------------------------------------
 */

static int
GetUserRecoverChannel(Tcl_Interp *interp, Burt_RecoverControlBlock *recoverCB)
{
    char *userProc;
    int mode;
    char *outputChannelStr;

    userProc = Tcl_GetVar2(interp, "burt_recover_proc", 
			   recoverCB->type, 
			   TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG);
    if (userProc == NULL) {
	/*
	 * Tcl_GetVar2 already filled in the error, couldn't find
	 * the user defined procedure. 
	 */
	return(TCL_ERROR);
    }

    if (Tcl_VarEval(interp, userProc, " ", recoverCB->host, " ",
		    recoverCB->atom, " ", recoverCB->level, 
		    NULL) == TCL_ERROR) {
	/* 
	 * Tcl_VarEval found an error in user proc and filled in the error.
	 */
	return(TCL_ERROR);
    }

    outputChannelStr = (char *) malloc(strlen(interp->result) + 1);
    if (outputChannelStr == NULL) {
	Tcl_SetResult(interp, "Out of memory in GetUserRecoverChannel",
		      TCL_STATIC);
	return(TCL_ERROR);
    } else {
	strcpy(outputChannelStr, interp->result);
    }

    recoverCB->outputChannel = Tcl_GetChannel(interp, outputChannelStr, &mode);
    if (recoverCB->outputChannel == NULL) {
	Tcl_ResetResult(interp);
	Tcl_AppendResult(interp, "can not find channel named \"",
			 outputChannelStr, "\" in user defined procedure",
			 userProc, (char *) NULL);
	free((void *) outputChannelStr);
	return(TCL_ERROR);
    
    }

    if ((mode & TCL_WRITABLE) == 0) {
	Tcl_ResetResult(interp);
	Tcl_AppendResult(interp, "channel ", outputChannelStr,
			 " wasn't opened for writing", 
			 " in user defined procedure ", 
			 userProc, (char *) NULL);
	Tcl_UnregisterChannel(interp, recoverCB->outputChannel);
	free((void *) outputChannelStr);
	return(TCL_ERROR);
    }     

    free((void *) outputChannelStr);
    return(TCL_OK);
}



/*
 *----------------------------------------------------------------------
 *
 * ProcessRecoverInputChannel --
 *
 *	This procedure processes input from (usually a tape drive)
 *      the inputChannel.  It compares the packet header read from the
 *      channel to the data in the RecoverControlBlock, until we find the 
 *      first block of the dump we are looking for, then we fill in the 
 *      outputChannel of the recoverCB and begin sending the buffer to the 
 *      outputChannel (which the user supplied via a Tcl procedure.)
 *
 * Results:
 *
 *
 * Side effects:
 *	
 *
 *----------------------------------------------------------------------
 */

static void 
ProcessRecoverInputChannel(ClientData clientData, int mask)
{
    Burt_RecoverControlBlock *recoverCB;
    Burt_PacketHeader header;
    char buffer[BURT_MAX_BLOCKSIZE];
    char logBuffer[1024];
    int nbytes;

    recoverCB = (Burt_RecoverControlBlock *) clientData;

    nbytes = BurtMtRead(recoverCB->inputChannel, &header,  buffer);

    if (nbytes == MT_EOT) {
        /* Get the next tape and get on with it already. */
        if (LoadNextRecoverTape(recoverCB->inputChannel, recoverCB->logChannel) == TCL_ERROR) {
	  Tcl_Write(recoverCB->logChannel, "BURT: I/O error during recover, couldn't load next tape.  I'm dying!\a\n", -1);
	  Tcl_Flush(recoverCB->logChannel);
	  Tcl_SetVar(recoverCB->interp, "burt_complete", "1", TCL_GLOBAL_ONLY);
	  return;
	}
	return;
    }
    
    if (nbytes == MT_ERROR) {
	/* MT_ERROR probably means a short read of the buffer */
	if (header.blockSize == END_OF_BACKUPSET) {
	    Tcl_DeleteChannelHandler(recoverCB->inputChannel, 
				     ProcessRecoverInputChannel, 
				     (ClientData) recoverCB);
	    Tcl_SetVar(recoverCB->interp, "burt_complete", "1", TCL_GLOBAL_ONLY);
	    return;
	}
	
	/* Try to read again.  If it was caused by EOT, there should be an EOT.
	 * If not, (ie the next MtRead does NOT return MT_EOT), it must have been
	 * a read error (?), so abort and bitch.
	 */
	if (BurtMtRead(recoverCB->inputChannel, &header, (void *) buffer) == MT_EOT) {
	    if (LoadNextRecoverTape(recoverCB->inputChannel, recoverCB->logChannel) == TCL_ERROR) {
	      Tcl_Write(recoverCB->logChannel, "BURT: I/O error during recover, couldn't load next tape.  I'm dying!\a\n", -1);
	      Tcl_Flush(recoverCB->logChannel);
	      Tcl_SetVar(recoverCB->interp, "burt_complete", "1", TCL_GLOBAL_ONLY);
	      return;
	    }
	    return;
	} else {
	    Tcl_Write(recoverCB->logChannel, "BURT: Read error during recover, aborting.\a\n", -1);
	    Tcl_Flush(recoverCB->logChannel);
	    Tcl_SetVar(recoverCB->interp, "burt_complete", "1", TCL_GLOBAL_ONLY);
	    return;
	}	    
    }
	
    if ((strcmp(recoverCB->host, header.host) == 0) 
	&& (strcmp(recoverCB->atom, header.atom) == 0) 
	&& ((strcmp(recoverCB->level, header.level) == 0) || (recoverCB->level[0] == '\0'))
	&& ((strcmp(recoverCB->type, header.type) == 0) || (recoverCB->type[0] == '\0'))) {

	if (recoverCB->outputChannel == NULL) {
	    strcpy(recoverCB->type, header.type);
	    strcpy(recoverCB->level, header.level);
	    if (GetUserRecoverChannel(recoverCB->interp, recoverCB) 
		== TCL_ERROR) {
		/*
		 * The error occured in the user defined function, and
		 * GetUserRecoverChannel() filled in the interp->result.
		 * Just kill the Channel handler and let Tcl report the
		 * error.
		 */
		Tcl_BackgroundError(recoverCB->interp);
		Tcl_DeleteChannelHandler(recoverCB->inputChannel, 
					 ProcessRecoverInputChannel, 
					 (ClientData) recoverCB);
		return;
	    }
	    sprintf(logBuffer, "BURT: Started %s %s %s %s\n", 
		    recoverCB->host, recoverCB->atom, recoverCB->level, recoverCB->type);
	    Tcl_Write(recoverCB->logChannel, logBuffer, strlen(logBuffer));
	}
	
	if (header.blockNum != ++(recoverCB->lastBlock)) {
	    sprintf(logBuffer, "BURT: Incorrect block sequence.  Got %d, expected %d.",
		    header.blockNum, recoverCB->lastBlock);
	    strcat(logBuffer, "  Aborting.\a\n");
	    Tcl_Write(recoverCB->logChannel, logBuffer, strlen(logBuffer));
	    Tcl_Flush(recoverCB->logChannel);
	    Tcl_SetVar(recoverCB->interp, "burt_complete", "1", TCL_GLOBAL_ONLY);
	    return;
	}

	/* 
	 * Check for end of backup, close up if we find it. 
	 */
	
	if (header.blockSize == END_OF_BACKUP) {
	    sprintf(logBuffer, "BURT: Completed %s %s %s %s\n",
		    header.host, header.atom, header.level, header.type);
	    Tcl_Write(recoverCB->logChannel, logBuffer, strlen(logBuffer));
	    Tcl_Flush(recoverCB->logChannel);
	    Tcl_DeleteChannelHandler(recoverCB->inputChannel, ProcessRecoverInputChannel, 
				     (ClientData) recoverCB);
	    Tcl_SetVar(recoverCB->interp, "burt_complete", "1", 
		       TCL_GLOBAL_ONLY);
	} else if (header.blockSize == ABORTED_BACKUP) {
	    sprintf(logBuffer, "BURT: %s %s %s %s was aborted.\a\n",
		    header.host, header.atom, header.level, header.type);
	    Tcl_Write(recoverCB->logChannel, logBuffer, strlen(logBuffer));
	    Tcl_Flush(recoverCB->logChannel);
	    Tcl_DeleteChannelHandler(recoverCB->inputChannel, ProcessRecoverInputChannel, 
				     (ClientData) recoverCB);
	    Tcl_SetVar(recoverCB->interp, "burt_complete", "1", TCL_GLOBAL_ONLY);
	} else if (header.blockSize == KILLED_BACKUP) {
	    sprintf(logBuffer, "BURT: %s %s %s %s was killed.\a\n",
		    header.host, header.atom, header.level, header.type);
	    Tcl_Write(recoverCB->logChannel, logBuffer, strlen(logBuffer));
	    Tcl_Flush(recoverCB->logChannel);
	    Tcl_DeleteChannelHandler(recoverCB->inputChannel, ProcessRecoverInputChannel, 
				     (ClientData) recoverCB);
	    Tcl_SetVar(recoverCB->interp, "burt_complete", "1", TCL_GLOBAL_ONLY);
	} else {
	    if (header.checksum != BurtChecksum((void *)buffer, header.blockSize)) {
		sprintf(logBuffer, "BURT: Incorrect checksum in block %d.  Aborting.\a\n", 
			header.blockNum);
		Tcl_Write(recoverCB->logChannel, logBuffer, strlen(logBuffer));
		Tcl_Flush(recoverCB->logChannel);
		Tcl_DeleteChannelHandler(recoverCB->inputChannel, ProcessRecoverInputChannel, 
					 (ClientData) recoverCB);
		Tcl_SetVar(recoverCB->interp, "burt_complete", "1", TCL_GLOBAL_ONLY);
		return;
	    }		    

	    nbytes = Tcl_Write(recoverCB->outputChannel, buffer, 
			       header.blockSize);
	    
	    if (nbytes < 0) {
		if (Tcl_GetErrno() == EPIPE) {
		    /*
                     * We don't consider a SIGPIPE an error due to
                     * the fact it happens all the time when /etc/restore
                     * finishes without reading all the data on tape.
                     */
		    Tcl_DeleteChannelHandler(recoverCB->inputChannel, 
					     ProcessRecoverInputChannel, 
					     (ClientData) recoverCB);
		    Tcl_SetVar(recoverCB->interp, "burt_complete", "1", 
			       TCL_GLOBAL_ONLY);
		} else { 
		    Tcl_SetErrno(nbytes);
		    Tcl_PosixError(recoverCB->interp);
		    Tcl_BackgroundError(recoverCB->interp);
		    Tcl_DeleteChannelHandler(recoverCB->inputChannel, 
					     ProcessRecoverInputChannel, 
					     (ClientData) recoverCB);
		    Tcl_SetVar(recoverCB->interp, "burt_complete", "1", 
			       TCL_GLOBAL_ONLY);
		}
	    }
	}
    }

    return;
}



/*
 *----------------------------------------------------------------------
 *
 * LoadNextRecoverTape --
 *
 *	This procedure prompts the user (via the logChannel) to mount the
 *      next tape, Before doing so it rewinds and unmounts the old one.  
 *      This procedure blocks until it gets a new tape and is able to read 
 *      a tape label. 
 *
 * Results:
 *	
 *      TCL_OK if the tape was swapped and the next tape label was
 *      read.
 *
 * Side effects:
 *      
 *----------------------------------------------------------------------
 */

static int 
LoadNextRecoverTape(Tcl_Channel mtChannel, Tcl_Channel logChannel)
{
    Tcl_File outHandle;   /* Tcl level generic file handle. */
    int fd;               /* Unix level file descriptor. */
    char logBuffer[1024]; /* Holds pretty log output */
    int bufferLength, i, success = 0;

    sprintf(logBuffer, "BURT: Error rewinding tape in backup.  Sleeping %d seconds...\a\n", TAPE_LOAD_DELAY);
    bufferLength = strlen(logBuffer);

    for (i = 0; i < 5; i++) {
      if (BurtMtRewind(mtChannel) == MT_ERROR) {
	Tcl_Write(logChannel, logBuffer, bufferLength);
	Tcl_Flush(logChannel);
	sleep(TAPE_LOAD_DELAY);
      } else {
	success = 1;
	break;
      }
    }

    if (success == 0)
      return (TCL_ERROR);

    success = 0;
    sprintf(logBuffer, "BURT: Error ejecting tape in backup.  Sleeping %d seconds...\a\n", TAPE_LOAD_DELAY);
    bufferLength = strlen(logBuffer);
    
    for (i = 0; i < 5; i++) {
      if (BurtMtOffline(mtChannel) == MT_ERROR) {
	Tcl_Write(logChannel, logBuffer, bufferLength);
	Tcl_Flush(logChannel);
	sleep(TAPE_LOAD_DELAY);
      } else {
	success = 1;
	break;
      }
    }

    if (success == 0)
      return (TCL_ERROR);

    outHandle = Tcl_GetChannelFile(mtChannel, TCL_READABLE);
    fd = (int) Tcl_GetFileInfo(outHandle, (int *)NULL);
    /*
     * SunOS pukes at the 2GB file size limit.  Because we hang on to
     * the file descriptor between tapes, reset the read file pointer
     * to zero so that we can write to the next tape (else the pointer is
     * negative because of 2GB limit and SunOS will never 'see' the new tape)
     */
    lseek(fd, 0, SEEK_SET);

    sprintf(logBuffer, "BURT: End of tape #%d in recover.\n", BurtGetBackupTapes());
    Tcl_Write(logChannel, logBuffer, strlen(logBuffer));
    Tcl_Flush(logChannel);

    BurtSetBackupTapes(BurtGetBackupTapes() + 1);
    sprintf(logBuffer, "BURT: Please mount tape #%d (%d seconds) in recover...\a\n",
	    BurtGetBackupTapes(), TAPE_LOAD_DELAY);
    bufferLength = strlen(logBuffer);

    do {
	Tcl_Write(logChannel, logBuffer, bufferLength); 
	Tcl_Flush(logChannel);
	sleep(TAPE_LOAD_DELAY);
    } while ((BurtMtRewind(mtChannel) == MT_ERROR) ||
	     (BurtMtReadLabel(mtChannel, NULL) == MT_ERROR));

    sprintf(logBuffer, "BURT: Mounted tape #%d in recover.  Thank you, have a nice day.\n",
	    BurtGetBackupTapes());
    Tcl_Write(logChannel, logBuffer, strlen(logBuffer));
    Tcl_Flush(logChannel);

    return(TCL_OK);
}

