/* hdup.c, a backup program
 *
 * Author: Miek Gieben
 *
 *
 * Copyright:
 *
 *   This package is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 dated June, 1991.
 * 
 *   This package is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License
 *   along with this package; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *   02111-1307, USA.
 *
 */

#include"hdup.h"
#include"prototype.h"

unsigned int quiet = 0;
char *scheme[] =  {"daily","weekly","monthly","remote","restore"};

void backup_remote(phost_t host[], int which, char *date) {

        /* get archive out of remote path */
        /* move to local global place */

        char *archive;

        /* set a lock */
        if (hdup_lock(host[which], LOCK) != 0 ) {
                /* could not get LOCK */
                hdup_cleanup(1, host[which]);
        }

        /* if remote is specified get the archive from the remote site
         * use the ssh options specified in the config file
         */
        if ( host[which]->remote != (char*)NULL ) {
                /* try to make the ->remote_path dir */
                hdup_mkdir(host[which]->remote_path, 0755, host[which]->user);
                if ( hdup_scp_from(host[which]) != 0 ) {
                        hdup_log("Could not retrieve to remote backup");
                        hdup_cleanup(-1,host[which]);
                } else {
                        hdup_overview("Scp",host[which]->remote);
                        if (host[which]->scp_opt != NULL )
                                hdup_overview("Scp options",host[which]->scp_opt);
                }
        }

        archive = (char*)malloc ( strlen(host[which]->name) + strlen(host[which]->remote_path) +
                        19 +    /* dots and consts */
                        8);     /* scheme */

        /* get everything thats adheres to this format to the
         * backupdir 
         */
        sprintf(archive, "%s/%s.*.*.tar.*", host[which]->remote_path,host[which]->name);

        /* check if the is an archive */
        if (hdup_globfilecheck(archive,NULL) == 0 ) {
                rmdir(host[which]->dirname_date);
                rmdir(host[which]->basename);
                hdup_log("No archive found");
                hdup_overview("Status", "no archive found");
                hdup_cleanup(-1, host[which]);
        }
        hdup_overview("Backup from",host[which]->remote_path);

        if (hdup_mv(archive,host[which]->dirname_date ) != 0 ) {
                /* cleanup our mess */
                rmdir(host[which]->dirname_date);
                rmdir(host[which]->basename);
                hdup_overview("Status","could not perform the move");
                hdup_cleanup(-1, host[which]);
        } 
        free(archive);
        rmdir(host[which]->dirname_etc);
        /* done !?!?! */
}

void backup_local(phost_t host[], int which, struct argument_t arg, struct option_t opt) {

        char *what;
        size_t l;

        /* fill in the rest */
        hdup_tarlist( host[which]);
        /* the exclude list */
        hdup_excludelist(host[which],host[0],opt.exclude);

        hdup_archive(host[which], arg.scheme);

        hdup_overview("Archive",(char*)hdup_lastname(host[which]->archivename));
        hdup_overview("User",host[which]->user);

        if ( host[which]->alg != NULL ) {
                l = strlen(host[which]->alg) + strlen(host[which]->keypath);
                what = (char*)malloc(l+10);
                sprintf(what,"yes (%s, %s)", host[which]->alg, host[which]->keypath);
                hdup_overview("Encryption", what);
        } else {
                hdup_overview("Encryption", "no");
        }

        hdup_inclist(host[which], arg.scheme);

        /* set a lock */
        if (hdup_lock(host[which], LOCK) != 0 ) {
                /* could not get LOCK */
                hdup_log("Could not get LOCK");
                hdup_cleanup(1, host[which]);
        }

        if ( hdup_tar(host[which]) != 0 ) {
                /* backup failed */
                hdup_log("Could not perform the backup");
                hdup_cleanup(1, host[which]);
        }
        if ( host[which]->alg != NULL ) 
                hdup_crypt(host[which]);

#ifdef DEBUG
        debug("remote",host[which]->remote);
#endif
        if ( host[which]->remote != NULL ) {
                /* scp to remote host 
                 * all the args are in ->scp_opt, the rest of the command
                 * should be in ->remote
                 */

                hdup_overview(NULL, NULL);
                if (host[which]->scp_opt != NULL ) {
                        /* user defined scp options */
                        hdup_overview("Scp",host[which]->remote);
                        hdup_overview("Scp options",host[which]->scp_opt);
                }

		/* Need this here, later archive is gone */
		hdup_overview("Archive size", (char*) (hdup_size (host[which]->archivename)));

                if ( hdup_scp(host[which]) != 0 ) {
                        /* hmmmm, something wrong */
                        if ( host[which]->remove == 1 )
                                hdup_log("Remote copy failed, not removing local copy");
                                hdup_cleanup(-1,host[which]);
                } else {
                        if ( host[which]->remove == 1 ) {
                                unlink(host[which]->archivename);
                                rmdir(host[which]->dirname_date); /* get ride of extra dir */
                        }
                }
        }
        /* done !?!? */
}

int main (int argc, char *argv[]) {

        phost_t host[MAXHOST + 1];
        struct option_t opt; struct argument_t arg;
        int c = 0; int which;

        /* reset options */
        opt.remove = opt.exclude = 0;
        opt.keypath     = NULL; opt.configpath  = NULL; 
        opt.alg         = NULL; 

        arg.scheme      = -1;   arg.host        = NULL;
        arg.date        = NULL; arg.extractdir  = NULL; 

        /* in host[0] I put the global options, these can be overriden
         * by the config file
         */
        while ( (c = getopt(argc, argv, "c:a:k:hrqve")) != -1) {
                 switch (c) { 
                         case 'a':
                                opt.alg = optarg;
                                break;
                         case 'k':
                                opt.keypath = optarg;
                                break;
                         case 'c':
                                opt.configpath = optarg;
                                break;
                         case 'h':
                                usage();
                                hdup_cleanup(-1,NULL);
                                break;
                         case 'v':
                                version();
                                hdup_cleanup(-1,NULL);
                         case 'e':
                                opt.exclude = 1;
                                break;
                         case 'r':
                                opt.remove = 1;
                                break;
                         case 'q':
                                quiet += 1;
                                break;
                         case '?':
                         default:
                                usage();
                                hdup_cleanup(-1,NULL);
                 }
        }

        argc -= optind;
        argv += optind;

        /* first the config file to set things up */
        /* -c option */
        if ( opt.configpath == NULL ) {
                hdup_log("No -c switch using default configfile \'/etc/hdup/hdup.conf\'");
                opt.configpath = ETCFILE;
        }
        if ( hdup_checkfile(opt.configpath) == 1  ) {
                /* there is a config file */
                if ( config(opt.configpath, host) != 0 ) {
                        /* something went wrong when reading
                         * the configfile
                         */
                        hdup_cleanup(-1,NULL);
                }
        }  else {
                /* there is no config file */
                hdup_log("No config file could be found");
                hdup_cleanup(-1,NULL);
        }

        /* config checks */
        if ( findhost("global",host) == -1 ) {
                hdup_log("No [global] section in configuration file");
                hdup_cleanup(-1,NULL);
        }

        /* -r option */
        host[0]->remove = opt.remove;
        /* -q option(s) */
        if ( quiet > 3 ) {
                /* specified more than 3 q, tsss */
                hdup_log("You can specify up to three -q's");
                hdup_cleanup(-1,NULL);
        }

        /* both must be set */
        if ( ( opt.alg == NULL && opt.keypath != NULL ) || ( opt.alg != NULL && opt.keypath == NULL ) ) {
                hdup_log("Both algorithm (-a) and keypath (-k) must be set");
                hdup_cleanup(-1,NULL);
        }
        if ( opt.alg != NULL ) {
                /* do we have mcrypt? */
                if (  hdup_checkfile (MCRYPT) == 0 ) {
                        hdup_log("Mcrypt not found, needed for encryption");
                        hdup_cleanup(-1,NULL);
                } else 
                        host[0]->alg = opt.alg;
                /* is key availible */
                if ( hdup_checkfile (opt.keypath) == 0 ) {
                        hdup_log("Encryption key not found");
                        hdup_cleanup(-1,NULL);
                } else
                        host[0]->keypath = opt.keypath; 
        }
        
        /* need at least two arguments: scheme and a hostname */
        if (argc < 2 ) {
                hdup_log("Not enough arguments");
                hdup_cleanup(-1,NULL);
        }

        /* arg: scheme */
        if ( strncmp (argv[0], scheme[DAILY], 6) == 0 )
                arg.scheme = DAILY;
        if ( strncmp (argv[0], scheme[WEEKLY], 7) == 0 ) 
                arg.scheme = WEEKLY;
        if ( strncmp (argv[0], scheme[MONTHLY], 8) == 0 ) 
                arg.scheme = MONTHLY;
        if ( strncmp (argv[0], scheme[REMOTE], 7) == 0 ) 
                arg.scheme = REMOTE;
        if ( strncmp (argv[0], scheme[RESTORE], 8) == 0 ) 
                arg.scheme = RESTORE;

        /* still -1? */
        if ( arg.scheme == -1 ) {
                hdup_log("Scheme must be: daily, weekly, monthly, remote or restore");
                hdup_cleanup(-1,NULL);
        }

        /* LOCAL */
        /* also allow for getting the archive from a remote host: 02 06 2002 */
        if ( arg.scheme <= REMOTE ) {
                /* second arg: host to backup */
                arg.host = argv[1];

                if ( argv[2] != NULL ) {
                        /* must start with @ */
                        if ( ( strchr(argv[2], '@')) == argv[2] ) {
                                /* do not copy the @ */
                                /* 29 jun 2002, store everything in ->remote */
                                host[0]->remote = ++argv[2];
                                /* check if there is a ':' in ->remote */
                                if ( hdup_grep (host[0]->remote, ':') == 1 ) {
                                        hdup_log("No colon found in @REMOTE_HOST");
                                        hdup_cleanup(-1,NULL);
                                }
                        } else {
                                        usage();
                                        hdup_cleanup(-1,NULL);
                        }
                }
        }

        /* REMOTE */
        if ( arg.scheme == REMOTE ) 
                /* REMOTE */
                arg.host = argv[1];

        /* RESTORE */
        if ( arg.scheme == RESTORE ) {
                /* restore a backup */
                if (argv[1] != NULL) 
                        arg.host = argv[1];
                else
                        hdup_log("You must supply a hostname"),hdup_cleanup(-1,NULL);

                if (argv[2] != NULL) 
                        arg.date = argv[2];
                else
                        hdup_log("You must supply a date (DD-MM-YYYY)"),hdup_cleanup(-1,NULL);
                if (argv[3] != NULL) {
                        arg.extractdir  = argv[3];
                        if ( hdup_globfilecheck(arg.extractdir,NULL) == 0 ) {
                                hdup_log("Extract dir doesn't exist");
                                hdup_cleanup(-1,NULL);
                        }
                        /* check for the / dir? */
                } else {
                        hdup_log("You must supply a dir to extract to");
                        hdup_cleanup(-1,NULL);
                }
        }

        /* check host in config file */
        which = findhost(arg.host, host);
        if ( which == -1 ) {
                hdup_log("Host not defined in configuration file");
                hdup_cleanup(-1,NULL);
        }

        /* ok ready, copy some vars over
         * conditional, can be overridden in config
         */
        if ( host[which]->tar_opt == (char*) NULL )
                host[which]->tar_opt = host[0]->tar_opt;

        /* check compression, and reset for use in the rest of the program  */
        /* looks a bit difficult, but this wil select gzip by default
         * and if not, it will take the default from the hdup.conf
         */
        if ( host[which]->tar_opt != NULL ) {
                if (  strcmp (host[which]->tar_opt, "gzip") == 0 ) {
                        /* ok, found */
                        host[which]->tar_opt    = TAR_GZIP;
                } else {
                        if ( strcmp(host[which]->tar_opt, "none") == 0 ) 
                                host[which]->tar_opt  = TAR_NONE;
                        if ( strcmp(host[which]->tar_opt, "bzip") == 0 ) 
                                host[which]->tar_opt  = TAR_BZIP;
                        if ( strcmp(host[which]->tar_opt, "old bzip") == 0 ) 
                                host[which]->tar_opt    = TAR_BZIPO;
                }
        } else 
               host[which]->tar_opt = TAR_GZIP; /* default */


        /* ok ready, copy some vars over
         * conditional, can be overridden in config
         */
        if ( host[which]->prerun == (char*) NULL )
                host[which]->prerun = host[0]->prerun;
        if ( host[which]->postrun == (char*) NULL )
                host[which]->postrun = host[0]->postrun;
        
        if ( host[which]->tar_opt == (char*) NULL )
                host[which]->tar_opt = host[0]->tar_opt;
        /* defaults are here */
        if ( host[which]->user == (char*) NULL ) {
                hdup_log("No user given defaulting to global user");
                if ( host[0]->user == (char*)NULL ) 
                        host[which]->user = "operator";
                else
                        host[which]->user = host[0]->user;
        }
        if ( host[which]->keypath == (char*) NULL )
                host[which]->keypath = host[0]->keypath;
        if ( host[which]->alg == (char*) NULL )
                host[which]->alg = host[0]->alg;
        if ( host[which]->remote == (char*) NULL )
                host[which]->remote = host[0]->remote;
        if ( host[which]->scp_opt == (char*) NULL )
                host[which]->scp_opt = host[0]->scp_opt;

        /* always copy these */
        host[which]->remove = host[0]->remove;
        host[which]->oldtar = host[0]->oldtar;

        /* the archive name is NOT known at this point
         * Pre run script starts here
         */
        if ( host[which]->prerun != (char*) NULL ) {
                /* we must do some prerun stuff */
                char *command = hdup_subst(host[which]->prerun,host[which]->name,"%h");
                if ( command != (char*)NULL ) {
                        if ( hdup_system(command) != 0 ) {
                                free(command);
                                /* something went wrong */
                                hdup_cleanup(-1,NULL);
                        }
                }
        }

        /* setup the overview status message */
        hdup_overview(NULL, NULL);
        hdup_overview("Program", PROGNAME);
        hdup_overview("Version", VERSION);
        hdup_overview(NULL,NULL);


        if ( arg.scheme == DAILY || arg.scheme == WEEKLY || arg.scheme == MONTHLY) {

                hdup_overview("Host", host[which]->name);
                hdup_overview("Date", host[which]->date);
                hdup_overview("Scheme", scheme[arg.scheme]);
                /* okay all the options are parsed we can do some REAL work */
                if ( host[which]->prerun != (char*)NULL ) 
                        hdup_overview("Pre script",host[which]->prerun);
                if ( host[which]->postrun != (char*)NULL ) 
                        hdup_overview("Post script",host[which]->postrun);
                
                hdup_setup(host, which);
                hdup_overview("Backup to",host[0]->path[0]);
                backup_local( host, which, arg, opt);
        } else if ( arg.scheme == REMOTE ) {
                /* fix hostname argument for REMOTE */
                host[which]->name = arg.host;
                hdup_overview("Host", host[which]->name);
                hdup_overview("Date", host[which]->date);
                hdup_overview("Scheme", scheme[arg.scheme]);
                /* make the dirs */
                hdup_setup(host, which);
                hdup_overview("Backup to",host[0]->path[0]);

                if ( host[which]->remote_path == (char*) NULL ) {
                        /* no remote path given */
                        hdup_log("No remote path specified, use \'remote = \' in hdup.conf");
                        hdup_cleanup(-1,NULL);
                }
                
                backup_remote(host, which, host[which]->date);
        } else if ( arg.scheme == RESTORE ) {
                char *arch;
                arch  = (char*)malloc(strlen(host[0]->path[0]) +
                                strlen(arg.host)+2);
                sprintf(arch,"%s/%s",host[0]->path[0],arg.host);
                if ( hdup_globfilecheck(arch,NULL) == 0 ) {
                        hdup_log("Cannot find archives for host");
                        hdup_cleanup(-1,NULL);
                }
                hdup_overview("Host", host[which]->name);
                hdup_overview("Date", host[which]->date);
                hdup_overview("Scheme", scheme[arg.scheme]);
                hdup_overview("Restore from",host[0]->path[0]);
                backup_restore(host, which, arg);
                hdup_cleanup(0,NULL); 
        }

        hdup_overview(NULL,NULL);
        if ( arg.scheme != REMOTE && host[0]->remote == NULL ) 
                hdup_overview("Archive size", (char*) (hdup_size (host[which]->archivename)));

        /* post run script */
        if ( host[which]->postrun != (char*) NULL ) {
                /* we must do some postrun stuff 
                 * a bit strange with the command, command2 ,etc.
                 * but alas...
                 */
                char *command = hdup_subst(host[which]->postrun,host[which]->archivename,"%a");
                if ( command == NULL )
                        command = hdup_subst(host[which]->postrun,host[which]->name,"%h"); 
                else {
                        char *tmp = hdup_subst(command,host[which]->name,"%h"); 
                        if ( tmp != NULL ) 
                                command = tmp;
                }

                if ( command != (char*)NULL ) {
                        if ( hdup_system(command) != 0 ) {
                                /* something went wrong */
                                free(command);
                                hdup_cleanup(-1,host[which]);
                        }
                }
        }

        hdup_overview("Status", "succesfully performed backup");
        hdup_cleanup(0,host[which]);

        return 0;
}
