/* hdupconf.c
 * parses a hdup config file
 * 
 * Author: Miek Gieben
 * Date: Wed, 10 Apr, 2002
 *
 * Changelog
 * 27-02-2002: started
 * 10-04-2002: reworked configfile
 * 07-07-2002: [global] must be the first host entry
 *
 * Copyright:
 *
 *   This package is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 dated June, 1991.
 * 
 *   This package is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License
 *   along with this package; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *   02111-1307, USA.
 *
 */

#include"hdup.h"
#include"prototype.h"

char *keyword[] =  {"dir","exclude", "remote","scp option", "user", "compression","prerun","postrun"};

int readline (char *line, FILE *from, int lim) {

        char *l = line;
        int c;

        while ( ( c = getc (from)) != EOF ) {

                if ( c != '\n' ) {
                        *l++ = c; lim--;
                } else {
                        *l = '\0'; return 0;
                }
                if ( lim < 0 )
                        return 1;
                
        }
        return 1;
}

/* todo: check for equal host names in the
 * same config file
 */
int config (char *path, phost_t host[]) {
        /* eats a list of host, and fills them up 
         * parse a hdup.conf2 file:
         * put the config file in a HOST struct
         */

        FILE *conf;        
        char *line     = (char *)malloc(MAXPATHLEN);
        char *left     = (char *)malloc(MAXPATHLEN);
        char *right    = (char *)malloc(MAXPATHLEN);
        char *h        = (char *)NULL;
        int i, len;

        len = 0; i = -1;

        conf = fopen (path, "r");
        if ( conf != NULL ) {
                char *date = (char *)malloc(11);
                while ( readline ( line, conf, MAXPATHLEN) == 0 ) {
                        len = strlen(line);

                        /* comment lines */
                        if ( *line == '#' || *line == '\n' || *line == '\0' ) 
                                continue; /* next line */

                        if ( *line == '[' ) {
                                /* new host found */
                                i++;
                                host[i] = (phost_t) malloc (sizeof(host_t));
                                host[i]->name = (char*) malloc(len + 1);

                                /* Solaris complains */
                                h = (char*)rindex(line,']');
                                /* not the first '[' */
                                line++;
                                /* and not the last */
                                strncpy(host[i]->name, line,  h - line );
                                *(host[i]->name + ( h - line) ) = '\0';
                                /* the first entry MUST be [global] 
                                 * hdup depends on this in the rest of the
                                 * program
                                 */
                                if ( i == 0 && ( strncmp(host[0]->name, "global",6) != 0 ) ) {
                                        hdup_log("First host in config file must be named [global]");
                                        hdup_cleanup(-1,NULL);
                                }
                                /* fill her up */
                                hdup_getdate(date);
                                host[i]->date           = date;   /* current date in DD-MM-YYYY */
                                host[i]->tar_opt        = NULL;   /* j=bzip g=inc */
                                host[i]->user           = NULL;   /* default user */
                                host[i]->basename       = NULL;
                                host[i]->dirname_date   = NULL;
                                host[i]->dirname_etc    = NULL;
                                host[i]->filelist       = NULL;
                                host[i]->excludelist    = NULL;
                                host[i]->inclist        = NULL;
                                host[i]->archivename    = NULL;
                                host[i]->keypath        = NULL;
                                host[i]->alg            = NULL;
                                host[i]->remote_path    = NULL;
                                host[i]->scp_opt        = NULL;
                                host[i]->remote         = NULL;
                                host[i]->prerun         = NULL;
                                host[i]->postrun        = NULL;

                                /* wouter bug fix: */
                                host[i]->path[0]        = NULL;
                                host[i]->exclude[0]     = NULL;
                                continue; /* next line */
                        }
                        if ( *line != '[' ) {
                                /* should be config line: blablabla = bliblibli
                                 * left part is keyword, right part is data 
                                 */
                                /* char* Solaris fix */
                                h = (char*)index(line,'=');
                                if ( h!= (char*)NULL ) {
                                        sscanf(line, "%256[^=]=%256[^\n]", left, right);
                                        
                                        /* get ride of spaces */
                                        chop(left);cut(right);
                                        switch (whatkey(left)) {
                                                case KEY_BACKUP:
                                                        setlist(right, host[i]->path);
                                                        break;
                                                case KEY_EXCLUDE:
                                                        setlist(right, host[i]->exclude);
                                                        break;
                                                case KEY_REMOTE:
                                                        host[i]->remote_path = (char*)malloc(strlen(right) + 3);
                                                        setvar(right, host[i]->remote_path);
                                                        break;
                                                case KEY_SCP_OPT:
                                                        host[i]->scp_opt = (char*)malloc(strlen(right) + 3);
                                                        setvar(right, host[i]->scp_opt);
                                                        break;
                                                case KEY_USER:
                                                        host[i]->user = (char*)malloc(strlen(right) +3);
                                                        setvar(right, host[i]->user);
                                                        break;
                                                case KEY_COMP:
                                                        host[i]->tar_opt = (char*)malloc(strlen(right)+3);
                                                        setvar(right, host[i]->tar_opt);
                                                        break;
                                                case KEY_PRE:
                                                        host[i]->prerun = (char*)malloc(strlen(right)+3);
                                                        setvar(right, host[i]->prerun);
                                                        break;
                                                case KEY_POST:
                                                        host[i]->postrun = (char*)malloc(strlen(right)+3);
                                                        setvar(right, host[i]->postrun);
                                                        break;
                                                default:
                                                        /* not a keyword */
                                                        hdup_log_nn("'");
                                                        hdup_log_nn(left);
                                                        hdup_log_nn("'");
                                                        hdup_log(" : is not a keyword");
                                                        return 1;
                                        }
                                }

                        }
                } /* end while */
                host[i+1] = (phost_t) malloc (sizeof(host_t));
                host[i+1]->name = (char*)NULL; /* possible error: i+1 */
        } else {
                /* oei */
                return 1;
        }
        return 0;
}

int findhost (char *hostname, phost_t host[]) {
        /* return -1 if non found, otherwise index from host */
        int i = 0;        

        while (host[i]->name != NULL) {
                if ( strcmp(host[i]->name, hostname) == 0 ) 
                        /* found! */
                        return i;
                i++;
        }
        return -1;
}

int setvar (char *dir, char *var) {
        /* set var to dir */

        int l;
        l = strlen(dir);
        if ( l > MAXPATHLEN )
                return -1;

        strcpy(var, dir);
        cut(var);
        return 0;
}

int setlist (char *dirpath, char *hostpath[]) {
        /* parse a comma seperate list of directories */
        int l;
        int t = 0;
        for(l = strlen(dirpath); l >= 0 ; l--) {
                if ( *(l+dirpath) == ',' ) {
                        hostpath[t] = (char*)malloc(strlen(dirpath) + 2 );
                        /* not the ',' */
                        strcpy(hostpath[t], dirpath + l + 1); 
                        cut(hostpath[t]);
                        *(l+dirpath) = '\0';
                        t++; continue;
                }
                if ( l == 0 ) {
                        hostpath[t] = (char*)malloc( strlen(dirpath) + 1 );
                        strcpy(hostpath[t], dirpath); 
                        /* may be optional */
                        cut(hostpath[t]);
                        t++;
                }
                if ( t > MAXDIR ) 
                        return -1;
        }
        /* still here */
        hostpath[t] = (char*)malloc(1); /*  don't forget to malloc! */
        hostpath[t] = '\0'; /* delimiter */
        return 0;
}

                
int whatkey(char *key) {
        /* found out what keyword is found */
        int i;
        for (i = 0; i < KEYWORDS; i++) {
                if ( strcmp ( key, keyword[i] ) == 0 ) {
                       return i;
                } 
        }
        return -1; /* no keyword found */
}

void chop (char *s) {
        /* chop of last whitespaces in a string */

        int len = strlen(s) ; /* to be right on the last char */
        while ( len > 0 ) {
                if ( isspace((int) *(s+len) ) || *(s+len) == '\n' || *(s+len) == '\0' ) {
                       *(s+len) = '\0';
                        len--;         
                } else
                        break;
        }
}

void cut (char *s) {
        /* cut of beginning white space */
        
        char *t = s;
        char c;
        int non_space = 0;

        while ( (c = *s++) != '\0' ) {
                /* order is important here */
                if ( !isspace((int) c) ) 
                        non_space = 1;
                if ( non_space == 1 ) 
                        *t++ = c;
        }
        *t = '\0';
}
