/* hduplib.c
 * lib with some general function
 * handy for hdup
 * 
 * Author: Miek Gieben
 * Date: Wed, 27 feb, 2002
 * 
 * Changelog
 *  27-02-2002: started
 *
 * Copyright:
 *
 *   This package is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 dated June, 1991.
 * 
 *   This package is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License
 *   along with this package; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *   02111-1307, USA.
 *
 */

#include"hdup.h"
#include"prototype.h"

char *dots   = "......................................";
char *spaces = "                                      ";
char *overview[OVERVIEW];

int hdup_getdate(char * d) {
        /* return the date in d
         * in the format:
         * DD-MM-YYYY
         * with 0-padding
         */

        time_t currenttime;
        struct tm *ltime;
        (void)time(&currenttime);
        ltime = localtime(&currenttime);

        snprintf(d,11, "%.2d-%.2d-%4d",ltime->tm_mday,
                          (ltime->tm_mon+1),
                          (ltime->tm_year+1900));
        return(0);
}

int hdup_setup ( phost_t host[], int which) {

        /* create /vol/backup/<hostname>
         *        /vol/backup/<hostname>/etc
         *        /vol/backup/<hostname>/<currentdate>
         */
        char *basename;
        char *d_etc;
        char *d_date;
        size_t baselen;

        /* 1 = \0 */
        baselen = strlen(host[0]->path[0])+ strlen(host[which]->name) +2;

        /* 2 = / \0 */
        basename = (char*) malloc ( baselen + 2);
        if (basename == NULL ) { hdup_log("Malloc error!!!"); hdup_cleanup(1,NULL);}
        /* create the path of the directory */
        strncpy ( basename, host[0]->path[0], strlen(host[0]->path[0]) );
        strcat ( basename, "/");
        strncat ( basename, host[which]->name, strlen(host[which]->name) );
        host[which]->basename = basename;

        /* 2 = / \0 */
        d_date = (char *) malloc (baselen + 11 + 2);
        if (d_date == NULL ) { hdup_log("Malloc error!!!"); hdup_cleanup(1,NULL);}
        /* create dir_date */
        strncpy ( d_date, basename, baselen);
        strcat  ( d_date, "/");
        strncat ( d_date, host[which]->date, strlen(host[which]->date));
        host[which]->dirname_date = d_date;

        /* 6 = / + 5 for '/etc\0' */
        d_etc = (char *) malloc ( baselen + 6);
        if (d_etc == NULL ) { hdup_log("Malloc error!!!"); hdup_cleanup(1,NULL);}
        /* create dir_etc */
        strncpy ( d_etc, basename, baselen);
        strcat  ( d_etc, "/etc");
        host[which]->dirname_etc = d_etc;

#ifdef DEBUG
        debug("basename",host[which]->basename);
        debug("etc",host[which]->dirname_etc);
        debug("date",host[which]->dirname_date);
#endif /*DEBUG*/
        (void)hdup_mkdir ( host[which]->basename , 0750, host[which]->user);
        (void)hdup_mkdir ( host[which]->dirname_etc, 0750, host[which]->user);
        (void)hdup_mkdir ( host[which]->dirname_date, 0750, host[which]->user);

        return 0;
}

int hdup_excludelist(phost_t host, phost_t glob, int exclude) {

        /* if exlude == 1 you should also put the global backup
         * dir in the excluded file list
         * this will prevent hdup from backing up it's on archive
         * which is pretty pointless
         */
        
        /* generate a file with the
         * patterns that should be 
         * excluded
         */

        FILE *fp;
        char *name;
        int i, printed_exclude;

        printed_exclude = 0;

        name = (char*)malloc (strlen(host->dirname_etc) + strlen(EXCLUDELIST) + 8);
        strncpy(name, host->dirname_etc, strlen(host->dirname_etc)+1 );
        strcat(name, EXCLUDELIST);
        host->excludelist = name;
        
        fp = fopen(host->excludelist, "w");
        if ( fp != NULL ) {
                /* ok, went alright */
                for (i = 0; i < MAXDIR; i++) 
                        if ( host->exclude[i] != NULL ) {
                                fprintf(fp,"%s\n",host->exclude[i]);
                                /* also write to overview */
                                if ( printed_exclude == 0 ) {
                                        hdup_overview("Excluding",host->exclude[i]);
                                        printed_exclude = 1;
                                } else {
                                        hdup_overview(host->exclude[i],NULL);
                                }
                        } else
                                /* no need to stay here
                                 * BUG fix. 19 jul
                                 */
                                break;
                /* if exclude is set, we also include the backup dir
                 * for hdup itself */
                if ( exclude == 1 ) {
                                fprintf(fp,"%s\n",glob->path[0]);
                                if ( printed_exclude == 0 )
                                        hdup_overview("Excluding",glob->path[0]);
                                else 
                                        hdup_overview(glob->path[0],NULL);
                }
        fclose(fp); 
        (void)hdup_chown ( host->excludelist, host->user);
        }
        return 0;

}

int hdup_tarlist (phost_t host) {

        /* fill a file with the dirs to tar
         * this file is given as an option
         * to TAR later on, this file is
         * called "filelist"
         * This file is put in
         * /vol/backup/<hostname>/etc/
         */
        FILE *fp;
        char *name;
        int i;

        name = (char*)malloc (strlen(host->dirname_etc) + strlen(FILELIST) + 8);
        strncpy(name, host->dirname_etc, strlen(host->dirname_etc)+1 );
        strcat(name, FILELIST);
        host->filelist = name;
        
        fp = fopen(host->filelist, "w");
        if ( fp != NULL ) {
                /* ok, went alright */
                for (i = 0; i < MAXDIR; i++) 
                        if ( host->path[i] != '\0' ) {
                                fprintf(fp,"%s\n",host->path[i]);
                                /* also write to overview */
                                if ( i == 0 ) 
                                        hdup_overview("Directories",host->path[i]);
                                else
                                        hdup_overview(host->path[i],NULL);
                        } else {
                                break;
                        }
        fclose(fp); 
        (void)hdup_chown ( host->filelist, host->user);
        }
        return 0;
}

int hdup_archive (phost_t host, int s) {
        /* fill hdup with the archive name */
              
        size_t len;
        len = strlen(host->dirname_date) + strlen(host->name) + 
                strlen(host->date) + strlen(scheme[s]);
        len = len + 15; /* extra space */

        /* /vol/backup/<host>/<date>/<host>.<date>.<scheme>.tar.bz2 */
        host->archivename = (char*) malloc(len);
        if ( strcmp (host->tar_opt, TAR_BZIPO) == 0 ) {
                hdup_overview("Compression","(old tar) bzip2");
                sprintf(host->archivename, "%s/%s.%s.%s%s",host->dirname_date,
                        host->name,host->date,scheme[s],BZIP_EXT);
                return 0;
        }
        if ( strcmp (host->tar_opt, TAR_GZIP) == 0 ) {
                hdup_overview("Compression","gzip");
                sprintf(host->archivename, "%s/%s.%s.%s%s",host->dirname_date,
                        host->name,host->date,scheme[s],GZIP_EXT);
                return 0;
        }
        if ( strcmp (host->tar_opt, TAR_NONE) == 0 ) {
                hdup_overview("Compression","none");
                sprintf(host->archivename, "%s/%s.%s.%s%s",host->dirname_date,
                        host->name,host->date,scheme[s],NONE_EXT);
                return 0;
        }
        /* default */
        hdup_overview("Compression","bzip2");
        sprintf(host->archivename, "%s/%s.%s.%s%s",host->dirname_date,
                host->name,host->date,scheme[s],BZIP_EXT);
        return 0;
}

int hdup_inclist (phost_t host, int s) {
        size_t len;

        char *inc_day, *inc_week, *inc_month = NULL;
        /* okay new idea */
        /*
         * month        -> delete month/week/day -> use month
         * weekly       -> if no weekly -> cp month to weekly
         *              | then use weekly + rm daily
         * daily        -> if no daily -> cp weekly to daily
         *              | then use daily
         */

        if ( s == REMOTE ) { 
                hdup_log("Internal error: wrong function call");
                hdup_cleanup(1, host);
        }

        /* len (/inclist.) = 9 */
        len = strlen(host->dirname_etc) + 9 + 8;

        inc_month = (char*) malloc(len);
        if ( inc_month == NULL ) 
                hdup_log("Internal error");
        snprintf(inc_month, len,"%s/inclist.%s",host->dirname_etc,scheme[MONTHLY]);

        inc_week = (char*) malloc(len);
        if ( inc_week == NULL ) 
                hdup_log("Internal error");
        snprintf(inc_week, len, "%s/inclist.%s",host->dirname_etc,scheme[WEEKLY]);

        inc_day = (char*) malloc(len);
        if ( inc_day == NULL ) 
                hdup_log("Internal error");
        snprintf(inc_day, len, "%s/inclist.%s",host->dirname_etc,scheme[DAILY]);

        switch (s) {
                case MONTHLY:
                        host->inclist = inc_month;
                        /* remove it */
                        unlink(host->inclist);
                        unlink(inc_week);
                        unlink(inc_day);
                        free (inc_week); free(inc_day);
                        break;
                case WEEKLY:
                        host->inclist = inc_week;
                        if ( hdup_checkfile(inc_month) == 1 ) {
                                /* does exist cp monthly to weekly */
                                (void)hdup_cp(inc_month,inc_week);
                                unlink(inc_day);
                        } else {
                                /* not usable */
                                hdup_overview("Status","could not find previous backup list");
                                rmdir(host->dirname_date);
                                hdup_cleanup(-2,host);
                        }
                        free(inc_month); free(inc_day);
                        break;
                case DAILY:
                        host->inclist = inc_day;
                        if ( hdup_checkfile(inc_week) == 1 ) {
                                /* does exist * cp weekly to daily */
                                (void)hdup_cp(inc_week,inc_day );
                        } else {
                                /* not usable */
                                hdup_overview("Status","could not find previous backup list");
                                rmdir(host->dirname_date);
                                hdup_cleanup(-2,host);
                        }
                        free(inc_month); free(inc_week);
                        break;
        }
        return 0;
}

int hdup_mv (char *from,char *to ) {
        int pid, status;

        char *command;
        command = (char*)malloc(strlen(from) + strlen(to) + 24 );

        /* check args */
        if ( from == (char*)NULL || to == (char*)NULL )
                return 1;

        sprintf(command, "/bin/mv -f %s %s 2>/dev/null", from, to);
                        
        if ((pid = fork()) < 0) {
                perror("fork");
                hdup_cleanup(1,NULL); /* MIEK hmm */
        }
        if ( pid == 0 ) {
                /* child */
                /* compose move */
               execl("/bin/sh", "sh", "-c",
                               command,
                               NULL);
               hdup_cleanup(1,NULL);
        }

        while (wait(&status) != pid)
                /* empty */ ;

        if ( status != 0 ) {
                /* shit hit the fan */
                return 1;
        }        
        return 0;
}

int hdup_cp (char *from, char *to ) {
        int pid, status;

        char *command;
        command = (char*)malloc(strlen(from) + strlen(to) + 27 );

        /* check args */
        if ( from == (char*)NULL || to == (char*)NULL )
                return 1;

        sprintf(command, "/bin/cp -f %s %s 2>/dev/null", from, to);
                        
        if ((pid = fork()) < 0) {
                perror("fork");
                hdup_cleanup(1,NULL);
        }
        if ( pid == 0 ) {
                /* child */
               execl("/bin/sh", "sh", "-c",
                               command,
                               NULL);
               hdup_cleanup(1,NULL);
        }

        while (wait(&status) != pid)
                /* empty */ ;

        if ( status != 0 ) {
                /* shit hit the fan */
                hdup_log("Could not perform the copy");
                return 1;
        }        
        return 0;
}

int hdup_lock(phost_t host, int l) {
        /* set a LOCK FILE 
         * in host->basename/LOCK
         */
        size_t len;
        char *lockfile;
        FILE *lock;

        /* /LOCK  = 5  + extra */
        len = strlen(host->basename);

        lockfile = (char *)malloc(len + 6);
        strcpy( lockfile, host->basename);
        strncat( lockfile, "/LOCK", 5);

#ifdef DEBUG
        debug("basename",host->basename);
        debug("lockfile",lockfile);
#endif

        switch (l) {
                case LOCK:
                        if (hdup_checkfile(lockfile) == 1 ) {
                                /* lockfile exists */
                                hdup_overview("Status", "LOCK found");
                                hdup_log("LOCK found");
                                hdup_cleanup(-1, NULL);
                        }
                        lock = fopen(lockfile, "w");
                        if ( lock != NULL) {
                                /* ok */
                                fclose(lock);
                        } else {
                                /*  hmm */
                                hdup_overview("Status","could not create LOCK");
                                hdup_log("Could not create LOCK");
                                hdup_cleanup(-1, NULL);
                                return 1;
                        }
                        break;
                case UNLOCK:
                        if (hdup_checkfile(lockfile) == 0 ) {
                                /* hmmm, nothing */
                                return 0;
                        } else {
                                /* lock found */
                                unlink(lockfile);
                        }
                        break;
        }
        return 0;
}

int hdup_scp_from (phost_t host) {
        /* get archive(s) from a remote machine 
         * and store them temporary in ->remote_path
         * then move them to [global]->path[0] for that
         * host
         */
        int status=0;
        char *scp_arg;
        char *remote_archive;

        remote_archive = (char*) malloc ( strlen(host->name) +
                        strlen(host->remote) + 12 );
        sprintf(remote_archive, "%s/%s.*.*.tar.*", host->remote, host->name);

        if (host->scp_opt == (char*) NULL ) {
                scp_arg = malloc ( strlen(remote_archive) +
                                strlen(host->remote_path) + 
                                strlen(MUTE) +
                                strlen(SCP) + 10);
                sprintf(scp_arg, "%s %s %s/",SCP, remote_archive, host->remote_path);
        } else {
                /* options in config */
                scp_arg = (char*)malloc (strlen(remote_archive) +
                        strlen(host->scp_opt) +
                        strlen(host->remote_path) +
                        strlen(MUTE) + 
                        strlen(SCP) + 10 );
                        /* + 4 for -i */
                sprintf(scp_arg, "%s %s %s %s/",SCP, host->scp_opt, remote_archive, host->remote_path);
        }
        /* quiet option */
        if ( quiet >= 1 ) {
                /* no subprocess output
                 * redirect to 2>/dev/null
                 */
                strcat(scp_arg,MUTE);
        }

#ifdef DEBUG 
        debug("scp_arg",scp_arg);
        debug("scp options", host->scp_opt);
#endif

        /* make it so... */
        status = system(scp_arg);
        free(scp_arg);
        if ( status != 0 ) {
                hdup_log("Could not perform the copy from remote system");
                return 1;
        } else
                return 0;
}

int hdup_system(char * command) {
        /* generic command executioner */
         
        int status = 0;

        char *my_command = (char*)malloc(strlen(command) + strlen(MUTE) + 2);
        strcpy(my_command, command);

        if ( quiet > 1 ) {
                strcat(my_command,MUTE);
        }
#ifdef DEBUG
        debug("my_command",my_command);
#endif
        status  = system(my_command);
        if ( status != 0 ) {
                hdup_log("Could not perform the command");
                return 1;
        } else 
                return 0;
}

int hdup_scp (phost_t host) {
        /* scp the archive to remote host:/tmp/backup */

        int status=0;
        char *scp_arg;

        if (host->scp_opt == (char*) NULL ) {
                scp_arg = (char*)malloc (strlen(host->remote) + 
                        strlen(host->archivename) + 
                        strlen(MUTE) + 
                        strlen(SCP) + 6 );
                sprintf(scp_arg, "%s %s %s",SCP, host->archivename, host->remote);
        } else {
                /* options in config */
                scp_arg = (char*)malloc (strlen(host->remote) + 
                        strlen(host->scp_opt) + 
                        strlen(host->archivename) + 
                        strlen(MUTE) + 
                        strlen(SCP) + 6 );
                        /* + 4 for -i */
                sprintf(scp_arg, "%s %s %s %s",SCP, host->scp_opt, host->archivename, host->remote);
        }
        /* quiet option */
        if ( quiet >= 1 ) {
                /* no subprocess output
                 * redirect to 2>/dev/null
                 */
                strcat(scp_arg,MUTE);
        }

#ifdef DEBUG 
        debug("scp_arg",scp_arg);
        debug("scp options", host->scp_opt);
#endif

        /* make it so... */
        status = system(scp_arg);
        free(scp_arg);
        if ( status != 0 ) {
                hdup_log("Could not perform the remote copy");
                return 1;
        } else 
                return 0;
}

int hdup_crypt (phost_t host) {
        /* crypt the archive */
        char *crypt_cmd;
        int status = 0;

#ifdef DEBUG
        debug("archive",host->archivename);
        debug("in hdup_crypt",NULL);
#endif
        /* mcrypt -a opt.alg -f opt.keypath host->archivename */
        crypt_cmd = (char*) malloc(strlen(MCRYPT) +
                strlen(host->alg) +
                strlen(host->keypath) +
                strlen(host->archivename) +
                strlen(MUTE) + 
                strlen("-u -a -f") + 5);
        sprintf(crypt_cmd,"%s -u -a %s -f %s %s",
                        MCRYPT,
                        host->alg,
                        host->keypath,
                        host->archivename);

        /* quiet option */
        if ( quiet >= 1 ) {
                /* no subprocess output
                 * redirect to 2>/dev/null
                 */
                strcat(crypt_cmd,MUTE);
        }
#ifdef DEBUG
        debug("crypt_cmd",crypt_cmd);
#endif
        /* make it so... */
        status = system(crypt_cmd);
        if ( status != 0 ) {
                hdup_log("Could not perform the encryption");
                return 1;
        } else {
                /* add the .nc extension */
                /* archivename length,....should be enough */
                strncat(host->archivename, MCRYPT_EXT, 3);
                (void)hdup_chown ( host->archivename, host->user);
                return 0;
        }
}


int hdup_tar (phost_t host) {
        /* perform the tar */
        char *tar_cmd; /* hold everything */
        int status = 0;

        /* tar cvfjg /tmp/back2.tar.bz2 inclist.SCHEME -T file-from */

        tar_cmd = malloc(strlen(TAR) +
                        strlen(host->tar_opt) +
                        strlen(host->archivename) +
                        strlen("--files-from") +
                        strlen(host->filelist) +
                        strlen("--listed-incremental") +
                        strlen(host->inclist) + 
                        strlen("--exclude-from") +
                        strlen(host->excludelist) +
                        strlen(MUTE) +          /* for the -q option */
                        strlen("--ignore-failed-read") + 15);
        sprintf(tar_cmd,"%s %s %s --files-from %s --listed-incremental %s --exclude-from %s --ignore-failed-read",
                        TAR, 
                        host->tar_opt, host->archivename,
                        host->filelist, host->inclist,
                        host->excludelist);

        /* quiet option */
        if ( quiet >= 1 ) {
                /* no subprocess output
                 * redirect to 2>/dev/null
                 */
                strcat(tar_cmd,MUTE);
        }

#ifdef DEBUG
        debug("tar_cmd",tar_cmd);
        debug("host_tar_opt",host->tar_opt);
        debug("host_filelist",host->filelist);
        debug("host_inclist",host->inclist);
#endif
                       
        /* make it so... */
        status = system(tar_cmd);
        if ( status != 0 ) {
                hdup_log("Error while tarring");
                return 1;
        } else {
                (void)hdup_chown ( host->archivename, host->user);
                (void)hdup_chown ( host->inclist, host->user);
                return 0;
        }
}

int hdup_untar (char *archive, char *extractdir, int compression) {
        char *tar_opt;
        char *tar_cmd;
        int status = 0;

        tar_opt = (char*)malloc(8);
        switch (compression) {
                case NONE:
                        tar_opt = "xpfC";
                        break;
                case GZIP:
                        tar_opt = "xpfCz";
                        break;
                case BZIP:
                        tar_opt = "xpfCj";
                        break;
                case BZIPO:
                        tar_opt = "xpfCI";
                        break;
                default:
                        tar_opt = "xpfCj";
                        break;
        }        
        tar_cmd = malloc(strlen(TAR) +
                strlen(tar_opt) +
                strlen(archive) +
                strlen(extractdir) +
                strlen(MUTE) +
                strlen("--incremental") +
                strlen("--ignore-failed-read") + 10);
        sprintf(tar_cmd,"%s %s %s %s --incremental --ignore-failed-read",
                        TAR,
                        tar_opt, 
                        archive,
                        extractdir);

        /* quiet option */
        if ( quiet >= 1 ) {
                /* no subprocess output
                 * redirect to 2>/dev/null
                 */
                strcat(tar_cmd,MUTE);
        }

#ifdef DEBUG
        debug("tar_cmd",tar_cmd);
        debug("tar_opt",tar_opt);
#endif
        /*
         * tar xvvvpfCz
         * /vol/backup/elektron/15-03-2002/elektron.15-03-2002.daily.tar.gz
         * /tmp/BLA --incremental
         */

        /* make it so... */
        status = system(tar_cmd);
        if ( status != 0 ) {
                hdup_log("Error while untarring");
                return 1;
        } else
                return 0;
}
 
int hdup_decrypt (char *archivename, phost_t host) {
        /* decrypt the archive */
        char *decrypt_cmd;
        int status = 0;

#ifdef DEBUG
        debug("archive",host->archivename);
#endif
        /* mdecrypt -a opt.alg -f opt.keypath >archivename */
        decrypt_cmd = malloc(strlen(MDECRYPT) +
                strlen(host->alg) +
                strlen(host->keypath) +
                strlen(archivename) +
                strlen(MUTE) + 
                strlen("-u -a -f") + 10);
        sprintf(decrypt_cmd,"%s -u -a %s -f %s %s",
                        MDECRYPT,
                        host->alg,
                        host->keypath,
                        archivename);

        /* quiet option */
        if ( quiet >= 1 ) {
                /* no subprocess output
                 * redirect to 2>/dev/null
                 */
                strcat(decrypt_cmd,MUTE);
        }
#ifdef DEBUG
        debug("decrypt_cmd",decrypt_cmd);
#endif
        /* make it so... */
        status = system(decrypt_cmd);
        if ( status != 0 ) {
                hdup_log("Could not perform the decryption");
                return 1;
        } else 
                return 0;
}

void hdup_log (char * message) {
        /* log with newline */
        if ( quiet < 2  || quiet > 3)
                fprintf(stderr, "%s\n", message);
}

void hdup_log_nn (char * message) {
        /* log without newline */
        if ( quiet < 2 || quiet > 3)
                fprintf(stderr, "%s", message);
}

void hdup_cleanup (int exitcode, phost_t host) {
        /* exit and do some cleanups */
        
        
        if ( host != NULL ) 
               hdup_lock(host, UNLOCK);

        /* newline */
        hdup_overview(NULL, NULL);

        /* no explicit status error */
        if ( exitcode > 0 ) 
                        hdup_overview("Status", "An error occured");
        
        /* do not print overview */
        if ( exitcode == -1 )
                        exit(1);

        /* explicit status error 
         * already put in overview
         */
        if ( exitcode == -2 )
                        exitcode = 2;

        hdup_print();
        exit (exitcode);
}

int hdup_overview (char *info, char *value) {
        
        /* fill the overview */
        int li, lv, i;
        static int row_count;

        if (row_count > OVERVIEW ) {
                hdup_log("overview overflow");
                hdup_cleanup(1, NULL);
        }

        if ( info == NULL && value == NULL ) {
                /* insert newline */
                overview[row_count] = (char*) malloc(2);
                *overview[row_count] = '\0';
                strncpy(overview[row_count], "\n\0",2);
                row_count++;
                return 0;
        }

        if ( value == NULL ) {
                /* just a message */
                li = strlen(info);
                i = DOTEND + 3;
                overview[row_count] = (char*) malloc(li+i+10);
                *overview[row_count] = '\0';

                strncpy(overview[row_count], spaces, DOTEND+3);
                strncat(overview[row_count], info, li);
                strncat(overview[row_count], "\n\0", 2);
        } else if ( info == NULL ) {
                /* illegal [MIEK] */
        } else {
                /* a info.....: value message */
                li = strlen(info); lv = strlen(value);
                i = DOTEND - li;
                overview[row_count] = (char*) malloc(li+lv+i+10);
                *overview[row_count] = '\0';

                strncpy(overview[row_count], info, li+1);
                strncat(overview[row_count], dots, i);
                strncat(overview[row_count], ":  ", 3);
                strncat(overview[row_count], value, lv+1);
                strncat(overview[row_count], "\n", 1);

        }
        row_count++;
        return 0;
}

void hdup_print (void) {
        int i = 0;

        if ( quiet != 3 ) {
                /* no 3 -q's */
                while ( overview[i] != (char*)NULL ) {
                        printf("%s", overview[i]);
                        i++;
                }
        }
}

char * hdup_size (char *filepath) {

        /* give back size in file,
         * 5 k, 5M, 5G 
         */
        off_t size; /* size in bytes */
        float s = 0;
        char suffix = '\0';
        char *filesize = (char*)malloc(6);
        struct stat *buf;
        buf = (struct stat*) malloc(sizeof(struct stat));

        if ( filepath == (char*)NULL) 
                return (char*)NULL;

        if ( stat(filepath, buf) == 0 ) {
                size = buf->st_size;

                if ( size > GIGABYTE ) {
                        s = (float) size / (float) GIGABYTE;
                        suffix = 'G';
                        sprintf(filesize,"%.1f%c",s, suffix);
                        return filesize;
                }
                else if (size > MEGABYTE) {
                        s = (float) size / (float) MEGABYTE;
                        suffix = 'M';
                        sprintf(filesize,"%.1f%c",s, suffix);
                        return filesize;
                }
                else if (size > KILOBYTE) {
                       s = (float) size / (float) KILOBYTE;
                       suffix = 'k';
                       sprintf(filesize,"%.0f%c",s, suffix);
                       return filesize;
                }
                sprintf(filesize,"%d%c",(int)size, suffix);
                free(buf);
                return filesize;
        }
        return (char*)NULL;
}

char * hdup_lastname (char * filepath) {
        char *w, *last, *t;

        if ( filepath == (char*)NULL) 
                return (char*)NULL;

        t = strrchr (filepath, '/');
        if ( t != NULL ) {
                /* advance 1 further */
                t++;
                last = (char*) malloc (strlen(filepath));
                w = last;
                while ( *t != '\0' ) {
                        *last = *t;
                        last++; t++;
                }
                *last = '\0';
                last = w; /* reset */
                return last;
        }
        return (char*)NULL;
}

int hdup_checkfile (char *filepath) {
        /*
         * returns * 1: file
         * doesn't do globbing
         */
         
        struct stat statbuf;

        if ( filepath == (char*)NULL) 
                return 0;

        if ( stat(filepath, &statbuf) != -1 ) 
                return (S_ISREG(statbuf.st_mode));

        return 0;
}

int hdup_checkdir ( char *dirpath) {

        struct stat statbuf;

        if ( dirpath == (char*)NULL) 
                return 0;

        if ( stat(dirpath, &statbuf) != -1 ) 
                return(S_ISDIR(statbuf.st_mode));

        return 0;
}

int hdup_globfilecheck( char *globpath, char *firstmatch) {
        /* use glob(3) to search for files */

        /* returns the number of found matches */

        int i, rg;
        glob_t globbuf;

        if (globpath == (char*)NULL ) 
                return 0;
                
        rg = glob(globpath,0,NULL,&globbuf);
        if ( rg == 0 ) {
                /* we have a match */
                i = globbuf.gl_pathc;
                if ( firstmatch != NULL && i != 0 ) {
                        strcpy(firstmatch, globbuf.gl_pathv[0]);
                }

                globfree(&globbuf);
                return 1;
        } else {
                globfree(&globbuf);
                return 0;
        }
}

int hdup_checkperm (char *filepath) {
        return 0;
}

int hdup_mkdir(char *dirpath, mode_t mode, char *user) {

        char *t, *d;
        size_t l = strlen(dirpath);

        if ( dirpath == (char*) NULL ) 
                return -1;

        d = (char *)malloc(l+2);
        strncpy(d,dirpath, l);

        /* quardian */
        if ( *(d+l) != '/' ) 
                *(d+l) = '/';

        /* we can go 1 step too far with t++ */
        for ( t = d ; *t != '\0' && (int)(t-d) <= l; t++ ) {
                if ( *t == '/' ) {
                        *t = '\0'; /* make mkdir think the strings stop here */
                        mkdir(d, mode);
                        if ( user != NULL ) 
                                hdup_chown(d,user);
                        *t = '/'; /* go on with the loop */
                }
        }
        return 0;
}

int hdup_chown (char *path, char *user) {

        struct passwd *entry;
        uid_t uid; gid_t gid;

        if  ( user == (char*) NULL ) 
                /* no user given */
                return -1;

        if ( ( entry = getpwnam(user)) != NULL ) {
                gid = entry->pw_gid;
                uid = entry->pw_uid;
                return ( chown(path, uid, gid));
        } else {
                return -1;
        }
}

int hdup_compression (char *archivename) {
        /* return the encryption use
         * and an archive
         */
        if ( archivename == NULL )
                return -1;

        /* check for BZIP */
        if (strstr(archivename, BZIP_EXT) != NULL )
                return BZIP;
        if (strstr(archivename, GZIP_EXT) != NULL )
                return GZIP;
        /* should be none than */
        return NONE;
}

int hdup_grep ( char *into, char c) {
        /* search for specific character in into */
        char *spot;

        if ( into == NULL ) 
                return -1;

        spot = index(into, c);

        if ( spot == (char*) NULL ) 
                /* not found */
                return 1;
        else
                return 0;
}

char * hdup_subst ( char *into, char *what, char *id) {
        /* substitute string what in the string into
         * where string id is located
         * returns pointer to the newly created string 
         * or *NULL
         */
        char *lwhere, *rwhere;
        char *left, *right; /* the newly formed substrings */
        char *newstring;
        size_t lsize, rsize,id_len, into_len, what_len;

        /* are we called correctly */
        if ( into == (char*)NULL || what == (char*)NULL
                        || id == (char*) NULL )
                return (char*)NULL;

        id_len = strlen(id); into_len = strlen(into);
        what_len = strlen(what);

        lwhere = strstr(into, id); /* find where id is located */
        if (lwhere == NULL ) 
                /* return the original string */
                return NULL;

        /* left part before id */
        lsize = lwhere - into; 
        left = (char*)malloc(lsize+1);
        strncpy(left,into,lsize);
        *(left + lsize) = '\0'; /* terminator */

        /* right part after id */
        rsize = into_len - ( lsize + id_len) ;
        right = (char*)malloc(rsize + 1);
        rwhere = into + into_len - rsize;
        strncpy(right, rwhere, rsize);
        *(right + rsize) = '\0'; /* terminiator */

#ifdef DEBUG
        debug("into",into);
        debug("what",what);
        debug("id",id);
        printf("left [%d]\n",lsize);
        printf("right [%d]\n",rsize);
        printf("[%s]\n",left);
        printf("[%s]\n",right);
        printf("size into: %d\n",into_len);
        printf("size what: %d\n",what_len);
        printf("size id: %d\n",id_len);
#endif
        
        /* get *what in between the two parts */
        newstring = (char*)malloc(lsize + rsize + what_len + 5);
        *newstring = '\0'; /* begin here */
        strncat(newstring, left, lsize);
        strncat(newstring, what, what_len);
        strncat(newstring, right, rsize);

#ifdef DEBUG
        printf("newstring: %s\n",newstring);
#endif
        return newstring;
}

int debug ( char *varname, char *varvalue) {
        /* print some simple debugging information */
        printf("[DEBUG] %s: [%s]\n",varname, varvalue);
        return 0;
}

int usage() {
        fprintf(stderr,"Usage:\n");
        fprintf(stderr,"\thdup [ OPTION ] SCHEME HOST [ @REMOTE_HOST:REMOTE_DIR ] (1st format)\n");
        fprintf(stderr,"\thdup [ OPTION ] restore HOST DATE DIRECTORY  (2nd format)\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"Arguments:\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"\tSCHEME");
        fprintf(stderr,"\t\tdaily, weekly, monthly, remote or restore\n");
        fprintf(stderr,"\tHOST");
        fprintf(stderr,"\t\tspecify which host to backup\n");
        fprintf(stderr,"\t\t\tor which host to restore\n");
        fprintf(stderr,"\t@REMOTE_HOST:REMOTE_DIR\n");
        fprintf(stderr,"\t\t\twhere to transfer the archive to\n");
        fprintf(stderr,"\t\t\twhen SCHEME is remote the archive is pulled from REMOTE_HOST\n");
        fprintf(stderr,"\tDATE");
        fprintf(stderr,"\t\twhen restoring: restore up to this date\n");
        fprintf(stderr,"\tDIRECTORY");
        fprintf(stderr,"\twhen restoring: restore to this directory\n");

        fprintf(stderr,"\nOptions:\n\n");
        fprintf(stderr,"\t-c config\tlocation of the configuration file\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"\t-a algorithm\tspecify algorithm to use when encrypting the archive\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"\t-k keyfile\tspecify the file with the encyption key\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"\t-r\t\tremove the archive after transferring it to remote_host\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"\t-e\t\tput global backup path in the exclude list\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"\t-q\t\tsuppress output from subprocesses\n");
        fprintf(stderr,"\t-q -q\t\tsuppress logging output\n");
        fprintf(stderr,"\t-q -q -q\tno logging at all, even no overview message\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"\t-h\t\tthis help\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"\t-v\t\tshow hdup version\n");
        fprintf(stderr,"\n");
        return 0;
}
 
int version() {
        /* print the version of hdup to stderr */
        fprintf(stderr,"%s %s\n",PROGNAME,VERSION);
        fprintf(stderr,"Written by Miek Gieben.\n\n");
        fprintf(stderr,"Copyright (C) 2001,2002 Miek Gieben. This is free software.\n");
        fprintf(stderr,"There is NO warrenty; not even for MERCHANTABILITY or FITNESS\n");
        fprintf(stderr,"FOR A PARTICULAR PURPOSE.\n");
        return 0;
}
