/* hduprestore.c
 * restore a filesystem stored with hdup
 * 
 * Author: Miek Gieben
 * Date: Tue, 4 apr, 2002
 *
 * Copyright:
 *
 *   This package is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 dated June, 1991.
 * 
 *   This package is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License
 *   along with this package; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *   02111-1307, USA.
 *
 */

#include"hdup.h"
#include"prototype.h"

/* we get a segfault is there is only 1 daily in an archive
 * normally you can not get in to that situation with hdup,
 * but this bug will get fixed
 */
void backup_restore(phost_t host[], int which, struct argument_t arg) {
        /* hostname
         * date
         * todir
         */
        char *restore[MAXRESTORE+1];
        int i=0;

        if ( host[which]->alg != NULL ) {
                char *what; size_t l;
                l = strlen(host[which]->alg) + strlen(host[which]->keypath) + 15;
                what = (char*)malloc(l);
                sprintf(what,"yes (%s, %s)", host[which]->alg, host[which]->keypath);
                hdup_overview("Encryption", what);
                free(what);
        } else {
                hdup_overview("Encryption", "no");
        }

        if (find_restore(arg,host[0],restore) == 0 ) {
                printf("No archives found: nothing restored\n");
                /* no restore archives found... */
                hdup_cleanup(-1,NULL);
        } else {
                hdup_overview("Archives",hdup_lastname(restore[0]));
                /* first see if there are monthly or weekly's to be done */
                
                while ( restore[i] != NULL  ) {
                        if ( i > 0 )
                                hdup_overview(hdup_lastname(restore[i]),NULL);

                        if ( host[which]->alg != NULL ) 
                                hdup_decrypt(restore[i],host[which]);

                        switch (hdup_compression(restore[i])) {
                        case NONE: 
                                   hdup_untar(restore[i],arg.extractdir,NONE);
                                   break;
                        case GZIP: 
                                   hdup_untar(restore[i],arg.extractdir,GZIP);
                                   break;
                        case BZIP: 
                                   if ( host[which]->oldtar == 0 ) {
                                        hdup_untar(restore[i],arg.extractdir,BZIP);
                                   } else { 
                                        hdup_untar(restore[i],arg.extractdir,BZIPO);
                                   }
                                   break;
                                }
                        i++;
                }
                hdup_overview(NULL,NULL);
                hdup_overview("Status","succesfully restored backup");
        }
}

int find_restore(struct argument_t arg, phost_t global, char *restore[]) {
        /* restore strategy:
         * look for first weekly and monthly backup
         * backwards in time
         *
         * return # found archives
         */
        int t,p,h,i,j;char *d;
        int mfound,wfound,last,tfound;
        char *directory,*backupdir;
        char *archivename;char *matchname;
        char *restore_loc[MAXRESTORE+1];

        backupdir = global->path[0];
        p = strlen(backupdir); h = strlen(arg.host);
        d=(char*) malloc(12);
        directory = (char*)malloc( p+ h + 11 + 5);

        /* make sure its long enough 
         * 2 times host: h+h, 2 times date: 12+12, scheme.tar.* */
        archivename = (char*) malloc(strlen(directory) + h + 12 + 8 + 15 +40);
        /* should be enough */
        matchname   = (char*) malloc(strlen(directory) + h + 12 + 8 + 15 +40);

        strncpy(d,arg.date,11);
        strncpy(directory,global->path[0],strlen(global->path[0]));
        i = 2; mfound = wfound = -1; tfound = 0;
        restore[0] = (char*)NULL;
        restore[1] = (char*)NULL;
       
        /* go back MAXRESTORE days  */
        for (t=0;t<=MAXRESTORE;t++) {
              sprintf(directory,"%s/%s/%s",backupdir,arg.host,d);
              if ( mfound == 1 && wfound == 1 ) { 
                      /* we should have them all */
                      break;
              }

              if (hdup_checkdir(directory) == 1) {
                      /* Only the daily's that come before the
                       * weekly's was found!!!
                       */
                      sprintf(matchname,"%s/%s.%s.%s.tar.*", directory,arg.host,d,scheme[DAILY]);
                      if ( hdup_globfilecheck(matchname,archivename) == 1) {
                               
                              restore_loc[i] = (char*) malloc(strlen(archivename)+2);
                              strcpy(restore_loc[i],archivename);
                              i++;tfound++;
                      }

                      /* First find the weekly */
                      sprintf(matchname,"%s/%s.%s.%s.tar.*", directory,arg.host,d,scheme[WEEKLY]);
                      if ( hdup_globfilecheck(matchname,archivename) == 1 && wfound == -1 ) {
                              /* there is a weekly in this dir */
                              /* we should not have the monthly */
                              if ( mfound == 1 ) 
                                      printf("Weekly archive found before any monthly, not good\n");
                              restore[1] = (char*) malloc(strlen(archivename)+2);
                              strcpy(restore[1],archivename);
                              wfound = 1;
                              tfound++;
                      }

                      /* Then the previous monthly */
                      sprintf(matchname,"%s/%s.%s.%s.tar.*", directory,arg.host,d,scheme[MONTHLY]);
                      if ( hdup_globfilecheck(matchname,archivename) == 1 && mfound == -1 ) {
                              /* there is a monthly in this dir */

                              restore[0] = (char*) malloc(strlen(archivename)+2);
                              strcpy(restore[0],archivename);
                              mfound = 1;
                              /* we have the monthly */
                              tfound++;
                      }

              }
              d = day(d,PREV);
        }

        /* guardian at the end */
        last = i - 1; j = 2;
        restore[i] = (char *)NULL;
        restore_loc[i] = (char*)NULL;
        if ( last > 1 ) {
                /* reverse the daily's */
                for (i = last; i >= 2; i-- ){
                        restore[j] = restore_loc[i];
                        j++;
                }
        } else {
                hdup_log("No daily's found");
                return tfound;
        }
        /* return total found archives */
        return tfound;
}

char * day(char *date, int direction ) {
        /* substract one day from date
         * and return the result
         */
        struct tm *ltime = (struct tm*)malloc(sizeof(struct tm));
        time_t abstime;
        char *mydate = malloc(11);
        int day,month,year;

        /* maybe some more checking...buffer overflow? */
        sscanf(date,"%2d-%2d-%4d",&day,&month,&year);
        ltime->tm_mday = day;
        ltime->tm_mon  = (month -1);
        ltime->tm_year = (year - 1900);

        ltime->tm_sec = 0; ltime->tm_min = 0;
        ltime->tm_hour = 0; ltime->tm_wday = 0;
        ltime->tm_yday = 0; ltime->tm_isdst = 0;

        abstime = mktime(ltime);        
        /* substract a day 
         * if direction is NEXT (+1) this add one day
         * if direction is PREV (-1) this substracts one day
         */
        abstime = abstime + ( direction * SECDAY );

        ltime = localtime(&abstime);

        snprintf(mydate,11, "%.2d-%.2d-%4d",ltime->tm_mday,
                          (ltime->tm_mon+1),
                          (ltime->tm_year+1900));
        return mydate;
}
