#!/bin/sh
#
# $Id: install.sh,v 1.1.1.1 2000/12/29 21:26:22 donncha Exp $
#
# Install Backup, by Donncha O Caoimh.
# This script will install and configure TOB, Tape Orientated Backup.
# TOB makes backups of specified directories. The script installs it and
# also sets up a cronjob to perform the backups every day.
#
# It will do a full backup on the day you install it, and incremental
# backups every other day. Backups are rotated every week.
#

clear;

printf "InstallBackup, Simple script to automate backups to disk.\n";
printf "Donncha O Caoimh <donncha@linux.ie>\n";
printf "Released under the GPL, 29th December 2000\n\n";
printf "Uses TOB to perform backups\n\n";

echo I\'m only going to ask you two questions. shouldn\'t take long.. 
echo

echo You need to tell me when to run the backup.
echo There are two things to consider:
echo "   Do it at a time when your machine won't be busy."
echo "   Do not run a backup at night if your machine is in your room."
echo 
FINISH=0
while [ $FINISH -eq 0 ]
do
  echo "Please enter the hour you want to run a backup and press RETURN. [1-24]"
  read HOUR
  if [ "$HOUR" != "" ] ; then
    if [ $HOUR -gt 0 ] ; then
      if [ $HOUR -lt 25 ] ; then
        FINISH=1
      fi
    fi
  fi
done

grep dailybackup /etc/crontab > /dev/null
if [ $? -eq 0 ] ; then
  echo "   Found an entry for dailybackup in /etc/crontab already."
  echo "   Please edit that file manually."
  echo "   No modifaction made to crontab."
  echo -------/etc/crontab-------
  cat /etc/crontab
  echo -------/etc/crontab-------
else 
  echo "   About to add the following line to /etc/crontab"
  echo "   0 $HOUR \* \* \* /usr/sbin/dailybackup"
  echo "   Press RETURN to continue, or CTRL-C to quit."
  read echo 0 $HOUR \* \* \* /usr/sbin/dailybackup >> /etc/crontab
fi

echo 
echo Where do you want to place your backups? The directory will be created.
echo DEFAULT: /mnt/win_d/backup
read BACKUPDIR
echo
if [ "$BACKUPDIR" = "" ] ; then
  BACKUPDIR="/mnt/win_d/backup"
fi
echo About to create $BACKUPDIR, /etc/tob.rc and other system files.
echo Press RETURN to continue, or CTRL-C to quit.
read

printf "Copying tob to /usr/sbin:               ";
cp -f tob/tob /usr/sbin/
printf "Done.\nCopying dailybackup to /usr/sbin:       ";
cp -f dailybackup /usr/sbin/
printf "Done.\nMaking directory /usr/etc/tob/volumes:  ";

mkdir -p /usr/etc/tob/volumes
printf "Done.\nMaking directory /usr/etc/tob/lists:    ";
mkdir -p /usr/etc/tob/lists

printf "Done.\nCopying config files:                   ";
cp -f volumes/* /usr/etc/tob/volumes/
sed s.BACKUPDIRECTORY.$BACKUPDIR. tob.rc > tob.tmp
dayofweek=`date '+%w'`
sed s.dayofbackup.$dayofweek. tob.tmp > /etc/tob.rc
rm -f tob.tmp
printf "Done.\nMaking directory $BACKUPDIR:  ";
mkdir -p $BACKUPDIR
printf "Done.\n\n";

echo 1. Edit the files in /usr/etc/tob/volumes/ to change which directories
echo    are backed up and which files are excluded.
echo 2. Edit /etc/tob.rc to change where backed up files go.
echo 3. Look in tob/doc/ for documentation on configuring tob. The defaults 
echo    should be ok for most people.
echo 4. You\'ll need to do a full backup first.
echo    Run /usr/sbin/dailybackup --full to do a full backup.
echo
echo Mail me at donncha@linux.ie if this works and saves you some day when
echo you delete some valuable files..

