#!/usr/bin/perl -w
#
#  WiredBackup 0.2 (incremental backup via ftp)
#
#  Copyright (C) 2001 Marcus Schiesser <webmaster@unc.de>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
use strict;
use Net::FTP;
my $program = "wiredbackup";
my $conffile = "/etc/wiredbackup.conf";
my ($server, $serverdir, $tempdir, $username, $ftppassword, $gpgpassword, $logfile);
my ($maxcount, @dirs, $snapshotfile, $backupfile, $suffix, $shutdown, $passive);
open(CONF, "<$conffile") or die "Cannot open config file";
while(<CONF>) { eval $_; }
close(CONF);

sub connect {
    # connect to ftp
	my $ftp = Net::FTP->new($server, Debug => 1, Passive => $passive);
	return undef if not defined $ftp;
	return undef if !$ftp->login($username, $ftppassword);
	if(!$ftp->cwd($serverdir)) {
		$ftp->mkdir($serverdir);
		$ftp->cwd($serverdir);
	}
	return $ftp;
}

sub ftpexists {
	my ($ftp, $file) = @_;
	my @entries = $ftp->ls();
	foreach my $entry (@entries) {
		return 1 if $file eq $entry;
	}
}

sub backup {
	my $ftp = &connect();
	return 0 if not defined $ftp;
	my $count = -1;
	my $ret = -1;
	# retrieve next free backupfile
	foreach (0..$maxcount-1) {
		if (!&ftpexists($ftp, "$backupfile.$_.$suffix")){
			$count = $_;
			last;
		}
	}
	# unsure tmp files are deleted
	my $tempfile = "$tempdir/$backupfile.$suffix";
	unlink("$tempdir/$snapshotfile");
	unlink($tempfile);
	unlink("$tempdir/$backupfile.$count.$suffix");
	if($count == 0) {
		# there are no backup files, so ensure a full dump
		# delete snapshotfile
		$ftp->delete($snapshotfile);
	}
	if($count == -1) {
		# there are too many incremental tars so please make a full dump
		# delete snapshotfile
		$ftp->delete($snapshotfile);
		# delete backupfiles
		my @backups = grep { $_="$backupfile.$_.$suffix" } (0..$maxcount-1);
		foreach my $file (@backups) {
			$ftp->delete($file);
		}
		$count = 0;
	}
	&log("getting actual snapshot file\n");
	# get actual snapshot file
	binary $ftp;
	$ftp->get($snapshotfile, "$tempdir/$snapshotfile");
	quit $ftp;
    # backupfile
	&log("invoking tar\n");
	if(system("/bin/tar cjv -g $tempdir/$snapshotfile -f $tempfile @dirs") != 0) {
		# backup error
		$ret = 0;
		&log("TAR error: $!\n");
		#syslog('error', 'tar error');
		goto cleanup;
	}
	# encrypt backupfile
	&log("invoking gpg\n");
	if(!open(PIPE,"| /usr/bin/gpg --output $tempdir/$backupfile.$count.$suffix --passphrase-fd 0 --symmetric $tempfile")) {
		&log("encryption error: $!\n");
		$ret = 0;
		goto cleanup;
	}
	print PIPE "$gpgpassword\n";
	close(PIPE);
	# upload backupfile
	&log("uploading tar...\n");
	$ftp = &connect();
	if (defined $ftp) {
		binary $ftp;
		$ftp->put("$tempdir/$backupfile.$count.$suffix", "$backupfile.$count.$suffix");
		$ftp->put("$tempdir/$snapshotfile", $snapshotfile);
		quit $ftp;
	} else {
		$ret = 0;
	}
	# clean up temp files
	cleanup:
	&log("cleaning up\n");
	unlink("$tempdir/$snapshotfile");
	unlink($tempfile);
	unlink("$tempdir/$backupfile.$count.$suffix");
	$ret;
}

sub shutdown {
	system("/sbin/shutdown -h now");
}

sub log {
	my ($log)=@_;
	my $time = localtime;
	print LOG "$time $log";
}

#openlog($program, 'cons,pid', 'user');
#syslog('info', 'started');
open(LOG, ">>$logfile");
&log("$program started...\n");
&backup();
close(LOG);
#closelog();
&shutdown() if $shutdown;

